def temp-table t field f as int.
def temp-table g field f as int.

def var i as int.
def var j as int.

output to tte.txt.

do i = 1 to 10:
   create g.
   assign g.f = i.
end.

B1:
repeat transaction on endkey undo, next
       on error undo, leave:
   
   i = 5. 
   /* 
    * After the client crashes during block B3 execution (see below for details), 
    * block B2 will try to rollback all created records - but the record with
    * t.f = 5 is already deleted by block B3, and so is not available. This
    * will result in a Hibernate exception to be thrown which in turn will
    * cause the table temporary table not to be droped. 
    */
   B2:
   for each g where g.f = i no-lock:
      
      do j = 1 to 5:
         create t.
         assign t.f = j.
      end.
      
      /* 
       * by placing a break-point in method TransactionManager.iterate() on 
       * line 2428 and after that closing the client, a RuntimeException will 
       * be thrown on the second iteration. This RuntimeException is caused by
       * the fact that the unnamed stream will be aware of the client crash,
       * when will try to make remote client call.
       * At the time of the break-point, P2J will already have committed the 
       * delete and so this delete will not be rolledback by block B3 (as the
       * commit was already executed). The problem is that block B2 has 
       * a CREATE call for the same record and will try to undo the CREATE (by
       * deleting the record) - but the record was already deleted by block B3
       * and so a Hibernate exception will be thrown.
       */
      B3:
      for each t where t.f <= i exclusive-lock by t.f desc:
         /* 
          * use a redirected frame, so when the block iterates the stream will 
          * be aware of the client crash
          */
         if t.f mod 2 = 0 then
            display t.f.
         delete t.
      end.
      
      /* 
       * the same abend is thrown when trying to rollback block B2: all the 
       * record being already deleted, block B2 doesn't have any records to 
       * rollback (i.e. delete), and so Hibernate will throw an exception. This
       * proves that the parent block must be aware if a certain record was
       * deleted in a child block - and, if it wants to rollback the CREATE
       * for that record, will not execute it.
       */
      undo, leave.
   end.

   leave.
end.

output close.