/*
** Module   : character.java
** Abstract : Progress 4GL compatible character object
**
** Copyright (c) 2004-2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20050517   @21295 Created initial version with full support
**                           for all Progress character processing
**                           including the concatination, begins, matches
**                           operators (but not CONTAINS which is not
**                           needed at this time), all builtin functions,
**                           case-sensitivity and the unknown value.
** 002 GES 20050606   @21424 Minor additions for function support.
** 003 GES 20050621   @21440 Added an assign() method to modify this
**                           instance's state based on the result of
**                           an expression of compatible type.
** 004 GES 20050627   @21554 Added replace/overlay methods to provide the
**                           substring/overlay assignment type language
**                           statements.
** 005 GES 20050628   @21565 Fixed typo in method name and added unknown
**                           value processing to toStringMessage().
** 006 SVG 20050720   @21768 Cloning support for BaseDataType added.
** 007 NVS 20050728   @21908 Added implementation of PROGRESS builtin fun-
**                           ction CAN-DO as public static canDo, taking
**                           either one or two parameters. One parameter
**                           version is just a catcher for untranslated
**                           CAN-DO calls, throwing an exception. Two par-
**                           ameter version is a real code which is typic-
**                           ally used as a utility function.
** 008 GES 20050728   @21920 Added support for the _cbit() built-in 
**                           function (see the testBitAt() method). Also
**                           changed the name of canDo to matchesList
**                           which is more generic and less like a
**                           Progress security-oriented function.
** 009 GES 20050815   @22118 Added Undoable interface support.
** 010 GES 20050825   @22253 Fixed toStringExport() to double up double
**                           quotes appearing inside the string.
** 011 GES 20050826   @22270 Implement case-sensitivity in output from
**                           specific functions and operators depending
**                           on the case-sensitivity of character operands
**                           or parameters.
** 012 GES 20050828   @22279 Added support for temporarily setting
**                           case-sensitivity with an automatic reset at
**                           the end of the scope in which the setting
**                           was made.
** 013 GES 20050829   @22284 Split assignment of configuration and state
**                           from normal assign(). deepAssign() is now 
**                           provided to set both configuration and state
**                           at the same time.
** 014 GES 20050829   @22371 Added error condition support.
** 015 GES 20050907   @22594 Added "raw" type support to substring().
** 016 ECF 20051004   @22958 Implement hashCode() method. Required to be
**                           consistent with BaseDataType's implementation
**                           of equals() method.
** 017 GES 20051024   @23122 Added convertToSQLLike() method.
** 018 ECF 20051104   @23237 Added iterate() method to SensitivityResetter
**                           inner class. No-op implementation; required
**                           to satisfy expanded Finalizable interface.
** 019 GES 20060112   @23892 Added additional valueOf signatures for the
**                           literal forms of the string(), fill(),
**                           entry(), indexOf() and lastIndexOf()
**                           functions.
** 020 GES 20060114   @23903 Fixed substring (2nd parm was used wrong).
** 021 GES 20060206   @24297 Made compareTo() safe to process any object
**                           type (avoids ClassCastException).
** 022 GES 20060208   @24387 Expose a format string expansion helper as
**                           public and added a default format string
**                           getter.
** 023 GES 20060303   @24882 Don't escape the single quote char in Java 
**                           strings.
** 024 GES 20060312   @24985 New variants for overlay and replace to
**                           handle all possible combos with primitives.
** 025 ECF 20060322   @25203 Honor case sensitivity in indexOf() methods.
**                           Added new indexOf variant to which all others
**                           delegate.  Performs scan with proper case
**                           sensitivity setting.
** 026 GES 20060403   @25318 New variants for lookup().
** 027 GES 20060410   @25457 New variants for substring().
** 028 GES 20060417   @25552 Added default value generation.
** 029 GES 20060420   @25620 Added new signatures for right/leftTrim.
** 030 GES 20060422   @25650 Changed assignment style syntax backing
**                           methods to return "this" such that the result
**                           can be assigned back in the case of a 
**                           database field. Added entry statement 
**                           support.
** 031 GES 20060427   @25762 New signatures for trim().
** 032 GES 20060427   @25767 New signature for length().
** 033 GES 20060508   @26023 Fix for Progress pattern to regular 
**                           expression conversion routine
**                           (convertToRegEx).
** 034 ECF 20060510   @26062 Introduced a runtime variant of static method
**                           convertToSQLLike(). This variant accepts a
**                           character parameter.
** 035 GES 20060517   @26192 Match new Finalizable interface requirement.
** 036 GES 20060518   @26280 Safety code for negative start indexes in
**                           substring.
** 037 GES 20060525   @26530 Fix for asc() which needed a 0-based index.
** 038 ECF 20060613   @27243 Fix for asc(). Replaced implementation to
**                           convert the low order character byte to an
**                           unsigned integer.
** 039 GES 20060724   @28158 Added a forced assign() abstract method.
** 040 GES 20060807   @28471 Moved to externalizable interface to optimize
**                           network performance.
** 041 ECF 20060812   @28586 Added methods needed for database server-side
**                           function support.
** 042 GES 20060904   @29214 Added helpers to space-ify strings with an
**                           embedded null character or null char escape
**                           sequence.
** 043 GES 20060905   @29222 Added embedded null processing for the
**                           toString() and concat() variants. Fixed
**                           null byte processing for asc().
** 044 GES 20060905   @29228 Added safeDuplicate().
** 045 GES 20060907   @29282 Fixed substring processing in "raw" mode.
** 046 GES 20060908   @29346 Fixed lookup() to return 0 instead of -1 on
**                           failure.
** 047 ECF 20060908   @29384 Made constructors which accept a J2SE String
**                           null-safe. A null string argument results in
**                           the instance being set to unknown value.
** 048 GES 20061109   @31050 Added formatLength().
** 049 ECF 20070306   @32284 Added case-sensitive variants of equals and
**                           hashCode. These are equalsCaseSensitively and
**                           caseSensitiveHashCode. Also modified existing
**                           hashCode method to be more consistent with
**                           equals. The previous implementation was not
**                           consistent with equals for case-insensitive
**                           character instances.
** 050 GES 20070321   @32504 Fix for some forms of lastIndexOf() which
**                           were starting the backwards search from the
**                           beginning of the source string instead of the 
**                           end as it should have been. This affected
**                           all forms of the method which only have 2
**                           parameters. Also fixed replaceAll() which
**                           would fail to replace two back-to-back
**                           instances of the same target string (the
**                           first would be replaced and the first char
**                           of the second match would be dropped).
** 051 GES 20070926   @35243 Provide external control over 4GL compatible
**                           escape characters when processing non-runtime
**                           strings in progressSpacifyNull(). On Windows
**                           that code should not treat the backslash as
**                           an escape character.
** 052 GES 20080312   @37442 Fixes and documentation added to format
**                           string processing. Fill characters and
**                           padding were not always being handled
**                           properly. Data characters were not being
**                           copied into the right locations in the output
**                           string when non-trailing fill characters were
**                           present.
** 053 GES 20080414   @38012 Fixed chr/asc to support all non-DBCS
**                           charsets. Added fromExportString().
** 054 ECF 20080804   @39286 Added 2 new flavors of numEntries(). Needed
**                           to handle combinations of character and
**                           String parameters.
** 055 GES 20080909   @39766 Moved concat() to varargs. This makes it
**                           possible to merge multiple concat operator
**                           usage into a single method call, which is
**                           significantly more efficient and cleaner.
** 056 GES 20080911   @39788 Added assign(String) signature.
** 057 GES 20081002   @40033 Modified method names to shorten calling
**                           code.
** 058 SVL 20081224   @40985 valueOf(..) returns unknown character value
**                           if converted data is an unknown value.
** 059 ECF 20090212   @41296 Made class final.
** 060 GES 20090422   @41905 Converted to standard string formatting.
** 061 GES 20090424   @41968 Import change.
** 062 GES 20090706   @43106 Added signatures for overlay().
** 063 ECF 20090814   @43621 Fixed various methods which use delimiters. Added
**                           replaceEntry(double, character, String) variant.
** 064 ECF 20090816   @43659 Rolled back #059 (@41296).
** 065 GES 20110303          Moved maximum/minimum to varargs.
** 066 GES 20110422          Fixed case-sensitivity inheritance in the return
**                           value of concat().
** 067 SVL 20110622          Modified convertToSQLLike, added
**                           convertToSQLBegins, postprocessStringLiteral,
**                           toUpperCase(String, boolean) and
**                           substring(String, int).
** 068 GES 20110801          Modified substring() signatures to use NumberType.
** 069 CS  20121031          Added quoter built-in function logic
*/

package com.goldencode.p2j.util;

import java.io.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import com.goldencode.p2j.util.BaseDataType;
import com.goldencode.p2j.util.character;
import com.goldencode.util.*;

/**
 * A class that represents a Progress 4GL compatible character object whose
 * data is mutable.  All Progress language features related to character
 * types are supported including all functionality that can be accessed via
 * the character concatination operator and builtin Progress string
 * functions.  Normal character comparison operators are implemented in
 * the {@link CompareOps} class.
 * <p>
 * In addition to the implementation of the base string processing functions
 * and operators, this class hides the representation of the 
 * <code>unknown value</code> and properly maintains the processing of
 * <code>unknown value</code>. 
 * <p>
 * Format string related processing is handled in this class, see method
 * {@link #toString(String)}.
 * <p>
 * The following is the mapping of Progress language features to the
 * corresponding feature in this class:
 * <p>
 * <pre>
 * +                operator (concatination)       {@link #concat}
 * begins           operator                       {@link #begins} and {@link #_begins}
 * matches          operator                       {@link #matches} and {@link #_matches}
 * contains         operator                       not supported at this time
 * _cbit            function                       {@link #testBitAt}
 * asc              function                       {@link #asc}
 * can-do           function                       {@link #matchesList}
 * caps             function                       {@link #toUpperCase}
 * chr              function                       {@link #chr}
 * compare          function                       {@link #compare}
 * entry            function                       {@link #entry}
 * index            function                       {@link #indexOf}
 * lc               function                       {@link #toLowerCase}
 * left-trim        function                       {@link #leftTrim}
 * length           function                       {@link #length} and {@link #byteLength}
 * lookup           function                       {@link #lookup}
 * maximum          function                       {@link #maximum}
 * minimum          function                       {@link #minimum}
 * num-entries      function                       {@link #numEntries}
 * r-index          function                       {@link #lastIndexOf}
 * replace          function                       {@link #replaceAll}
 * right-trim       function                       {@link #rightTrim}
 * string           function                       {@link #valueOf}
 * substitute       function                       {@link #substitute}
 * substring        function                       {@link #substring}
 * trim             function                       {@link #trim}
 * entry            language statement             {@link #replaceEntry}
 * overlay          language statement             {@link #overlay}
 * substring        language statement             {@link #replace}
 * </pre> 
 * <p>
 * There are convenience versions of the begins and matches operators that
 * directly return <code>boolean</code> values such that unwrapping is not
 * needed.  These are named the same as the operator methods above except 
 * there is no 'With' appended.  For example, the begins operator is
 * {@link #begins}.
 * <p>
 * The following is the list of Progress language features which optionally
 * return a <code>character</code> instance that reflects the case-sensitivity
 * of the operands or input values:
 * <p>
 * <pre>
 * +                operator
 * caps             function
 * lc               function
 * left-trim        function
 * maximum          function
 * minimum          function
 * replace          function
 * right-trim       function
 * substring        function
 * trim             function
 * </pre>
 * <p>
 * Strangely enough, the <code>chr, entry</code> and <code>substitute</code>
 * functions DO NOT provide this support in Progress.  Each of the
 * corresponding backing methods properly duplicate this behavior.
 * <p>
 * <b>Please note that at this time, no conversion of Progress strings to
 * Java strings is done.  All strings passed are expected to be Java
 * strings.  This behavior may need changing in the case where a Progress
 * string is built at runtime but needs to be used as a Java string.</b> In
 * this case the {@link #progressToJavaString} method should be used.
 *
 * @author    GES
 */
public class character
extends BaseDataType
{
   /** Specifies the less than operator for {@link #compare}. */
   private static final int LT = 0; 
   
   /** Specifies the less than or equal operator for {@link #compare}. */
   private static final int LE = 1; 
   
   /** Specifies the greater than operator for {@link #compare}. */
   private static final int GT = 2; 
   
   /** Specifies the greater than or equal operator {@link #compare}. */
   private static final int GE = 3; 
   
   /** Specifies the equal operator for {@link #compare}. */
   private static final int EQ = 4; 

   /** Specifies the not equal operator for {@link #compare}. */
   private static final int NE = 5; 
   
   /** Specifies the RAW type for {@link #compare} or {@link #substring}. */
   private static final int RAW = 6; 
   
   /** 
    * Specifies the COLUMN type for {@link #compare} or {@link #substring}.
    */
   private static final int COLUMN = 7; 
   
   /** Specifies the FIXED type for {@link #compare} or {@link #substring}. */
   private static final int FIXED = 8; 
   
   /**
    * Specifies the CHARACTER type for {@link #compare} or {@link #substring}.
    */
   private static final int CHARACTER = 9; 
   
   /** Specifies the CASE-SENSITIVE strength for {@link #compare}. */
   private static final int CASE_SENSITIVE = 10; 
   
   /** Specifies the CASE-INSENSITIVE strength for {@link #compare}. */
   private static final int CASE_INSENSITIVE = 11; 
   
   /** The default format string. */
   private static final String defaultFormat = "x(8)";
   
   /** Charset name to use for encoding or <code>null</code> for default. */
   private static final String override = Utils.getCharsetOverride();
   
   /**
    * Stores the state of this character as an instance of class
    * <code>String</code>.  This is independent of the state of whether or
    * not this instance represents the <code>unknown value</code>.
    */
   private String value = null; 
   
   /**
    * Stores the state of whether or not this instance represents the
    * <code>unknown value</code>, independent of the state of this
    * character's data.
    */
   private boolean unknown = false;
   
   /**
    * Stores the state of whether or not this instance compares case
    * sensitively.
    */
   private boolean caseSens = false;
   
   /**
    * Default constructor, creates an instance that represents the unknown
    * value.
    */
   public character()
   {
      unknown = true;
   }
   
   /**
    * Constructs an instance that has the <code>String</code> value passed
    * in and which <b>does not</b> represent the <code>unknown value</code>,
    * unless the <code>value</code> argument is <code>null</code>.
    *
    * @param    value
    *           The character value of the resulting instance.
    */
   public character(String value)
   {
      if (value == null)
      {
         setUnknown();
      }
      else
      {
         this.value = value;
      }
   }
   
   /**
    * Constructs an instance that has the <code>String</code> value passed
    * in and which <b>does not</b> represent the <code>unknown value</code>
    * unless the <code>value</code> argument is <code>null</code>.
    *
    * @param    value
    *           The character value of the resulting instance.
    * @param    caseSens
    *           Specifies whether this instance should use case sensitive
    *           comparisons.
    */
   public character(String value, boolean caseSens)
   {
      if (value == null)
      {
         setUnknown();
      }
      else
      {
         this.value    = value;
         this.caseSens = caseSens;
      }
   }
   
   /**
    * Constructs an instance that has a <code>char</code> value passed
    * in and which <b>does not</b> represent the <code>unknown value</code>. 
    *
    * @param    ch
    *           The character value of the resulting instance.
    */
   public character(char ch)
   {
      this.value = new StringBuffer().append(ch).toString();
   }
   
   /**
    * Constructs an instance that has the <code>String</code> value and
    * <code>unknown value</code> that exactly matches that of the passed-in 
    * instance.
    *
    * @param    value
    *           The character and unknown value of the resulting instance.
    */
   public character(character value)
   {
      deepAssign(value);
   }
   
   /**
    * Returns the largest instance (lexicographically) from the list.  This
    * list may not include <code>null</code> values.  Since an instance
    * is returned which is one of the references that was passed in, the
    * case-sensitivity of the input will be the same on output.
    *
    * @param    list
    *           The list of character instances to compare.
    *
    * @return   The largest instance or <code>unknown value</code> if any
    *           entries in the list are unknown or <code>null</code> if
    *           an empty list is passed.
    */
   public static character maximum(character ... list)   
   {
      return (character) CompareOps.maximum(list);
   }
      
   /**
    * Returns the smallest instance (lexicographically) from the list.  This
    * list may not include <code>null</code> values.  Since an instance
    * is returned which is one of the references that was passed in, the
    * case-sensitivity of the input will be the same on output.
    *
    * @param    list
    *           The list of character instances to compare.
    *
    * @return   The smallest instance or <code>unknown value</code> if any
    *           entries in the list are unknown or <code>null</code> if
    *           an empty list is passed.
    */
   public static character minimum(character ... list)   
   {
      return (character) CompareOps.minimum(list);
   }   
   
   /**
    * Implements the <code>+</code> character concatination operator which 
    * returns an instance of the same class.  If any operand is the
    * <code>unknown value</code>, the result is <code>unknown value</code>.
    * If any operand is case-sensitive, the result will be case-sensitive.
    * <p>
    * It is valid to pass a mixture of <code>BaseDataType</code> and other
    * object types to this method.  Types that are not <code>character</code>
    * will not affect the case-sensitivity or unknown status of the result.
    * 
    * @param    list
    *           The operands.
    *
    * @return   The result of the operator.
    */
   public static character concat(Object ... list)
   {
      StringBuilder sb = new StringBuilder();
      
      boolean caseSens = false;
      
      for (Object txt : list)
      {
         if (txt instanceof character)
         {
            character op = (character) txt;
            
            // any character operand that is unknown causes the entire
            // result to be unknown
            if (op.unknown)
            {
               return new character();
            }
            
            // any character operand that is case-sensitive causes the
            // entire result to be case-sensitive
            caseSens = (caseSens || op.caseSens);
            
            sb.append(op.safeValue());
         }
         else
         {
            sb.append(txt.toString());
         }
      }
      
      return new character(sb.toString(), caseSens);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, basing case sensitivity on the left operand.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>false</code> if there is no match or if either
    *           operand is <code>unknown</code>.
    */
   public static boolean _matches(character op, character pattern)
   {
      if (op.unknown || pattern.unknown)
         return false;
      
      return _matches(op.value, pattern.value, op.caseSens);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, basing case sensitivity on the left operand.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>false</code> if there is no match or if the left
    *           operand is <code>unknown</code>.
    */
   public static boolean _matches(character op, String pattern)
   {
      if (op.unknown)
         return false;
      
      return _matches(op.value, pattern, op.caseSens);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, case-insensitively.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>false</code> if there is no match or if the right
    *           operand is <code>unknown</code>.
    */
   public static boolean _matches(String op, character pattern)
   {
      if (pattern.unknown)
         return false;
      
      return _matches(op, pattern.value, false);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, ignoring the case of the operand and the
    * pattern.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression.
    */
   public static boolean _matches(String op, String pattern)
   {
      return _matches(op, pattern, false);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, honoring the case sensitivity preference of
    * the caller.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    * @param    caseSens
    *           <code>true</code> if a case-sensitive comparison should be
    *           used.
    *
    * @return   <code>true</code> if op matches the regular expression.
    */
   public static boolean _matches(String op, String pattern, boolean caseSens)
   {
      String up = caseSens ? op : op.toUpperCase();
      
      pattern = caseSens ? pattern : pattern.toUpperCase();
      return up.matches(character.convertToRegEx(pattern));
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, basing case sensitivity on the left operand.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>unknown value</code> if either operand is
    *           <code>unknown</code>.
    */
   public static logical matches(character op, character pattern)
   {
      if (op.unknown || pattern.unknown)
         return new logical();
      
      return matches(op.value, pattern.value, op.caseSens);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, basing case sensitivity on the left operand.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>unknown value</code> if the left operand is
    *           <code>unknown</code>.
    */
   public static logical matches(character op, String pattern)
   {
      if (op.unknown)
         return new logical();
      
      return matches(op.value, pattern, op.caseSens);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, case-insensitively.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression or
    *           <code>unknown value</code> if the right operand is
    *           <code>unknown</code>.
    */
   public static logical matches(String op, character pattern)
   {
      if (pattern.unknown)
         return new logical();
      
      return matches(op, pattern.value, false);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, ignoring the case of the operand and the
    * pattern.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    *
    * @return   <code>true</code> if op matches the regular expression.
    */
   public static logical matches(String op, String pattern)
   {
      return matches(op, pattern, false);
   }
   
   /**
    * Tests if the first operand matches a regular expression that is 
    * provided by the caller, honoring the case sensitivity preference of
    * the caller.
    * <p>
    * The Progress style matching pattern passed as a parameter is converted
    * to a regular expression using {@link #convertToRegEx} during this
    * method.
    *
    * @param    op
    *           String to check for a match.
    * @param    pattern
    *           Progress style matching pattern.
    * @param    caseSens
    *           <code>true</code> if a case-sensitive comparison should be
    *           used.
    *
    * @return   <code>true</code> if op matches the regular expression.
    */
   public static logical matches(String  op,
                                 String  pattern,
                                 boolean caseSens)
   {
      String up = caseSens ? op : op.toUpperCase();
      
      pattern = caseSens ? pattern : pattern.toUpperCase();
      boolean result = up.matches(character.convertToRegEx(pattern));
      
      return new logical(result);
   }
   
   /**
    * Converts a string (from a Progress source file) that represents a
    * Progress 4GL matching "pattern" to a traditional regular expression
    * which is suitable for a Java source file.  This allows a trivial
    * implementation of the matches operator using the
    * <code>String.matches</code> method.
    * <p>
    * In Progress 4GL, there are 2 escape characters '~' (tilde) and '\'
    * backslash.  In both Java and in regular expressions, the only escape
    * character is '\'.  In addition, since these strings come directly from
    * source files, the Progress runtime has not handled the escape sequences
    * on input as it normally would.  So although the sequences '~*' or '\*'
    * are normally the literal asterisk, the string must be coded to appear
    * as '~~*' or '\\*' in the source because the escape chars are normally 
    * dropped on input.  This method is written assuming that all of the
    * normal Progress conversions have already occurred by utilizing 
    * {@link #progressToJavaString} and {@link StringHelper#processEscapes}.
    * <p>
    * Once the standard Progress conversions have occurred, then this method
    * can be used and the following conversions will be additionally handled:
    * <pre>
    *    Progress                Regular Expression
    *    --------                ------------------
    *    ~.                      \\.
    *    \.                      \\.
    *    ~*                      \\*
    *    \*                      \\*
    *    ~~                      ~
    *    \~                      ~
    *    ~\                      \\\\
    *    \\                      \\\\
    *    *                       .*
    *    ~                       (removed or converted to \\ if the following char requires)
    *    \                       (removed or converted to \\ if the following char requires)
    * </pre>
    * <p>
    * The following characters in Progress have special meaning in the
    * extended regular expression syntax used in Java.  Each of these must
    * be properly escaped to ensure that their meaning is not accidentally
    * invoked by an input pattern that expects it to be matched literally:
    * <p>
    * <pre>
    *    Progress                Extended Regular Expression
    *    --------                ------------------
    *    ?                       \\\\?
    *    +                       \\\\+
    *    ^                       \\\\^
    *    $                       \\\\$
    *    |                       \\\\|
    *    (                       \\\\(
    *    )                       \\\\)
    *    [                       \\\\[
    *    ]                       \\\\]
    *    {                       \\\\{
    *    }                       \\\\}
    * </pre>
    * The following special Progress pattern constructs actually match those
    * of regular expressions and for this reason, these constructs are left
    * alone (not disturbed or modified):
    * <pre>
    *    Progress                Regular Expression
    *    --------                ------------------
    *    .                       .      
    * </pre>    
    * This conversion seems trivial, except that the backslash '\' character
    * must be handled properly since it needs to be doubled up for Java to
    * recognise one in a Java string and if two are needed in the regular
    * expression then one needs to double them up, twice.  This means that
    * in order to match a single \ in a regular expression, the Java string
    * will need \\\\ (4 backslashes).  In addition, since the
    * asterisk '*' character is a wildcard (it matches any number of any
    * character) in Progress, it must be properly replaced in the regular
    * expression (where .* is the unlimited wildcard).  Both Progress and
    * regular expressions use '.' to mean match any single character.
    *
    * @param   pattern
    *          Progress 4GL matching pattern.    
    *
    * @return  Converted regular expression.
    */
   public static String convertToRegEx(String pattern)
   {
      StringBuffer sb = new StringBuffer("");      
            
      int end = pattern.length();
      
      // convert the contents into the equivalent Java form
      for (int i = 0; i < end; i++) 
      {
         char current = pattern.charAt(i);
         char next;
         
         switch (current)
         {
            case '*':
               sb.append(".*");
               break;
            case '~':
            case '\\':
               // there must always be a following character
               i++;
               next = pattern.charAt(i);               
               
               // handle the special cases that don't have a one-to-one
               // correspondence in Java
               if (next == 'E')
               {
                  // the special case for the ESC char
                  sb.append("\\033");
               }
               else
               {
                  // only certain chars need to be AND in fact only these MAY
                  // BE escaped in Java so drop the preceding escape char
                  // unless the following char is in this list
                  if (next == 'b'  || next == 't'  || next == 'n' ||
                      next == 'f'  || next == 'r'  || next == '\''||
                      next == '\"' || next == '\\' || next == '.' ||
                      next == '*'  ||(next >= '0' && next <= '7'))
                  {
                     // we must convert the leading tilde into the Java
                     // escape char because all of these must be escaped in
                     // Java
                     sb.append('\\');
                  }

                  // insert our following char (warning: this is a bit funky
                  // for the octal escapes because we don't write the whole
                  // thing at once, but the following chars will all trip
                  // the default processing and be passed through so this
                  // works even if it isn't as obvious as we would like)
                  sb.append(next);                  
               }               
               break;            
            default:
               // certain chars need to have an escape inserted because they 
               // have no special meaning in Progress but do have a special
               // meaning in Java's extended regular expression syntax
               if (current == '?' || current == '+' || current == '^' ||
                   current == '$' || current == '|' || current == '(' ||
                   current == ')' || current == '{' || current == '}' ||
                   current == '[' || current == ']')
               {
                  // insert a leading escape char
                  sb.append("\\");
               }

               // not an escaped char or special case
               sb.append(current);
         }
      }
      
      return sb.toString();
   }
   
   /**
    * Converts a string (from within a character variable) that represents a
    * Progress 4GL matching "pattern" to a traditional SQL LIKE expression.
    * This variant is intended for runtime use.
    *
    * @param   pattern
    *          Progress 4GL matching pattern.    
    *
    * @return  Converted SQL LIKE expression, or unknown value if
    *          <code>pattern</code> represents the unknown value.
    *
    * @see     #convertToSQLLike(java.lang.String, boolean)
    */
   public static character convertToSQLLike(character pattern)
   {
      if (pattern.isUnknown())
      {
         return (new character());
      }
      
      return (new character(convertToSQLLike(pattern.getValue(), true)));
   }
   
   /**
    * Converts a string (from a Progress source file) that represents a
    * Progress 4GL matching "pattern" to a traditional SQL LIKE expression
    * which is suitable for a Java source file.  This allows a trivial
    * implementation of the matches operator using the SQL LIKE operator.
    * <p>
    * In Progress 4GL, there are 2 escape characters '~' (tilde) and '\'
    * backslash.  In both Java and in SQL, the only escape
    * character is '\'.  In addition, since these strings come directly from
    * source files, the Progress runtime has not handled the escape sequences
    * on input as it normally would.  So although the sequences '~*' or '\*'
    * are normally the literal asterisk, the string must be coded to appear
    * as '~~*' or '\\*' in the source because the escape chars are normally 
    * dropped on input.  This method is written assuming that all of the
    * normal Progress conversions have already occurred by utilizing 
    * {@link #progressToJavaString} and {@link StringHelper#processEscapes}.
    * <p>
    * Once the standard Progress conversions have occurred, then this method
    * can be used and the following conversions will be additionally handled:
    * <pre>
    *    Progress       SQL LIKE
    *    --------       -----------------
    *    ~.             .
    *    \.             .
    *    ~*             *
    *    \*             *
    *    ~~             ~
    *    \~             ~
    *    ~\             \\
    *    .              _
    *    *              %
    *    %              \\% (runtime); \\\\% (hardcoded)
    *    _              \\_ (runtime); \\\\_ (hardcoded)
    *    ~              (removed or converted to \\ if the following char requires)
    *    \              (removed or converted to \\ if the following char requires)
    * </pre>
    * This conversion seems trivial, except that the backslash '\' character
    * must be handled properly since it needs to be doubled up for Java to
    * recognise one in a Java string.
    *
    * @param   pattern
    *          Progress 4GL matching pattern.
    * @param   runtimeString
    *          <code>true</code> if the target string is a runtime string,
    *          <code>false</code> if it is a hardcoded string into a source
    *          file.
    *
    * @return  Converted SQL LIKE expression.
    */
   public static String convertToSQLLike(String pattern, boolean runtimeString)
   {
      StringBuffer sb = new StringBuffer("");      
            
      int end = pattern.length();
      
      // convert the contents into the equivalent Java form
      for (int i = 0; i < end; i++) 
      {
         char current = pattern.charAt(i);
         char next;
         
         switch (current)
         {
            case '*':
               sb.append("%");
               break;
            case '.':
               sb.append("_");
               break;
            case '~':
            case '\\':
               // there must always be a following character
               i++;
               next = pattern.charAt(i);               
               
               // handle the special cases that don't have a one-to-one
               // correspondence in Java
               if (next == 'E')
               {
                  // the special case for the ESC char
                  sb.append("\\033");
               }
               else
               {
                  // only certain chars need to be AND in fact only these MAY
                  // BE escaped in Java so drop the preceding escape char
                  // unless the following char is in this list
                  if (next == 'b'  || next == 't'  || next == 'n' ||
                      next == 'f'  || next == 'r'  || next == '\''||
                      next == '\"' || (next >= '0' && next <= '7') ||
                      next == '\\')
                  {
                     // we must convert the leading tilde into the Java
                     // escape char because all of these must be escaped in
                     // Java
                     sb.append('\\');
                  }

                  // insert our following char (warning: this is a bit funky
                  // for the octal escapes because we don't write the whole
                  // thing at once, but the following chars will all trip
                  // the default processing and be passed through so this
                  // works even if it isn't as obvious as we would like)
                  sb.append(next);                  
               }               
               break;
            case '%':
               sb.append(runtimeString ? "\\%" : "\\\\%");
               break;
            case '_':
               sb.append(runtimeString ? "\\_" : "\\\\_");
               break;
            default:
               // not an escaped char or special case
               sb.append(current);
         }
      }

      return sb.toString();
   }

   /**
    * Converts a second operand of the "op1 BEGINS op2" function into
    * corresponding SQL LIKE expression. It is assumed that output string is a
    * runtime string.
    * The following conversions will be performed:
    *  %    to    \\% (runtime); \\\\% (hardcoded)
    *  _    to    \\_ (runtime); \\\\_ (hardcoded)
    *  \\   to    \\\\
    * Also % will be appended to the end of the string.
    *
    * @param  str
    *         Represents a second operand of the "op1 BEGINS op2" function.
    *
    * @return target SQL LIKE expression.
    */
   public static character convertToSQLBegins(character str)
   {
      if (str.isUnknown())
      {
         return (new character());
      }

      return (new character(convertToSQLBegins(str.getValue(), true)));
   }

   /**
    * Converts a second operand of the "op1 BEGINS op2" function into
    * corresponding SQL LIKE expression.
    * The following conversions will be performed:
    *  %    to    \\% (runtime); \\\\% (hardcoded)
    *  _    to    \\_ (runtime); \\\\_ (hardcoded)
    *  \\   to    \\\\
    * Also % will be appended to the end of the string.
    *
    * @param  str
    *         Represents a second operand of the "op1 BEGINS op2" function.
    * @param  runtimeString
    *         <code>true</code> if the target string is a runtime string,
    *         <code>false</code> if it is a hardcoded string into a source
    *         file.
    *
    * @return The corresponding SQL LIKE expression. 
    */
   public static String convertToSQLBegins(String str, boolean runtimeString)
   {
      StringBuffer sb = new StringBuffer("");

      int end = str.length();

      for (int i = 0; i < end; i++)
      {
         char current = str.charAt(i);

         switch (current)
         {
            case '%':
               sb.append(runtimeString ? "\\%" : "\\\\%");
               break;
            case '_':
               sb.append(runtimeString ? "\\_" : "\\\\_");
               break;
            case '\\':
               sb.append("\\\\"); // extra escaping is added for hardcoded 
                                  // strings into postprocessStringLiteral
               break;
            default:
               sb.append(current);
         }
      }
      sb.append("%");
      return sb.toString();
   }

   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if either of the two operands is
    * case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>false</code> if there is no match or if either
    *           operand is <code>unknown value</code>.
    */
   public static boolean _begins(character op1, character op2)
   {
      if (op1.unknown || op2.unknown)
         return false;
      
      boolean cs = (op1.caseSens || op2.caseSens);
      
      return _begins(op1.value, op2.value, cs);
   }
   
   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if the right operand is case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>false</code> if there is no match or if the right
    *           operand is <code>unknown value</code>.
    */
   public static boolean _begins(String op1, character op2)
   {
      if (op2.unknown)
         return false;
      
      return _begins(op1, op2.value, op2.caseSens);
   }
   
   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if the left operand is case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>false</code> if there is no match or if the left
    *           operand is <code>unknown value</code>.
    */
   public static boolean _begins(character op1, String op2)
   {
      if (op1.unknown)
         return false;
      
      return _begins(op1.value, op2, op1.caseSens);
   }
   
   /**
    * Tests if the first operand begins with the second operand, ignoring the
    * case of the two operands.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2.
    */
   public static boolean _begins(String op1, String op2)
   {
      return _begins(op1, op2, false);
   }
   
   /**
    * Tests if the first operand begins with the second operand, honoring the
    * case sensitivity preference of the caller.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    * @param    caseSens
    *           <code>true</code> if a case-sensitive comparison should be
    *           used.
    *
    * @return   <code>true</code> if op1 begins with op2.
    */
   public static boolean _begins(String op1, String op2, boolean caseSens)
   {
      String up1 = caseSens ? op1 : op1.toUpperCase();
      String up2 = caseSens ? op2 : op2.toUpperCase();
      
      return up1.startsWith(up2);
   }
   
   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if either of the two operands is
    * case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>unknown value</code> if either operand is
    *           <code>unknown value</code>.
    */
   public static logical begins(character op1, character op2)
   {
      if (op1.unknown || op2.unknown)
         return new logical();
      
      boolean cs = (op1.caseSens || op2.caseSens);
      
      return begins(op1.value, op2.value, cs);
   }
   
   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if the right operand is case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>unknown value</code> if the right operand is
    *           <code>unknown value</code>.
    */
   public static logical begins(String op1, character op2)
   {
      if (op2.unknown)
         return new logical();
      
      return begins(op1, op2.value, op2.caseSens);
   }
   
   /**
    * Tests if the first operand begins with the second operand, implementing
    * a case-sensitive comparison if the left operand is case-sensitive.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2 or 
    *           <code>unknown value</code> if the left operand is
    *           <code>unknown value</code>.
    */
   public static logical begins(character op1, String op2)
   {
      if (op1.unknown)
         return new logical();
      
      return begins(op1.value, op2, op1.caseSens);
   }
   
   /**
    * Tests if the first operand begins with the second operand, ignoring the
    * case of the two operands.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    *
    * @return   <code>true</code> if op1 begins with op2.
    */
   public static logical begins(String op1, String op2)
   {
      return begins(op1, op2, false);
   }
   
   /**
    * Tests if the first operand begins with the second operand, honoring the
    * case sensitivity preference of the caller.
    *
    * @param    op1
    *           First string to compare.
    * @param    op2
    *           Second string to compare.
    * @param    caseSens
    *           <code>true</code> if a case-sensitive comparison should be
    *           used.
    *
    * @return   <code>true</code> if op1 begins with op2.
    */
   public static logical begins(String op1, String op2, boolean caseSens)
   {
      String up1 = caseSens ? op1 : op1.toUpperCase();
      String up2 = caseSens ? op2 : op2.toUpperCase();
      
      return new logical(up1.startsWith(up2));
   }
   
   /**
    * Tests if the current PROGRESS user is allowed in the list.
    * Since in the P2J environment the security subsystem is based on the
    * Security Manager, this call simply catches the overlooked CAN-DO call
    * in the PROGRESS code.
    *
    * @param    list
    *           comma-separated list of user IDs, possibly with ! and *
    *           modifiers.
    *
    * @return   never
    * @throws   IllegalArgumentException
    *           to catch the untranslated CAN-DO call
    */
   public static logical matchesList(character list)
   throws IllegalArgumentException
   {
      throw new IllegalArgumentException("untranslated CAN-DO call");
   }

   /**
    * Tests if the current PROGRESS user is allowed in the list.
    * Since in the P2J environment the security subsystem is based on the
    * Security Manager, this call simply catches the overlooked CAN-DO call
    * in the PROGRESS code.
    *
    * @param    list
    *           comma-separated list of user IDs, possibly with ! and *
    *           modifiers.
    *
    * @return   never
    * @throws   IllegalArgumentException
    *           to catch the untranslated CAN-DO call
    */
   public static logical matchesList(String list)
   throws IllegalArgumentException
   {
      throw new IllegalArgumentException("untranslated CAN-DO call");
   }

   /**
    * Tests if a string specified with the "item" parameter is allowed in the 
    * comma-separated list of allowed items.
    *
    * @param    list
    *           comma-separated list of allowed items, possibly with ! and *
    *           modifiers.
    * @param    item
    *           an item being tested
    *
    * @return   <code>true</code> if item is allowed in the list
    */
   public static logical matchesList(character list, character item)
   {
      if (list.unknown || item.unknown)
         return new logical();

      return matchesList(list.value, item.value);
   }

   /**
    * Tests if a string specified with the "item" parameter is allowed in the 
    * comma-separated list of allowed items.
    *
    * @param    list
    *           comma-separated list of allowed items, possibly with ! and *
    *           modifiers.
    * @param    item
    *           an item being tested
    *
    * @return   <code>true</code> if item is allowed in the list
    */
   public static logical matchesList(character list, String item)
   {
      if (list.unknown)
         return new logical();

      return matchesList(list.value, item);
   }

   /**
    * Tests if a string specified with the "item" parameter is allowed in the 
    * comma-separated list of allowed items.
    *
    * @param    list
    *           comma-separated list of allowed items, possibly with ! and *
    *           modifiers.
    * @param    item
    *           an item being tested
    *
    * @return   <code>true</code> if item is allowed in the list
    */
   public static logical matchesList(String list, character item)
   {
      if (item.unknown)
         return new logical();

      return matchesList(list, item.value);
   }

   /**
    * Tests if a string specified with the "item" parameter is allowed in the 
    * comma-separated list of allowed items.
    *
    * @param    list
    *           comma-separated list of allowed items, possibly with ! and *
    *           modifiers.
    * @param    item
    *           an item being tested
    *
    * @return   <code>true</code> if item is allowed in the list
    */
   public static logical matchesList(String list, String item)
   {
      boolean result = false;

      // create the regular expression (with proper escapes)
      String regex = fixupRegexChar(',');      
      
      // split up the list (make sure all possible entries are included,
      // even empty trailing entries --> use -1 as the limit to force this) 
      String[] array = list.split(regex, -1);
      
      // checking item against every entry in the list until match is found
      int idx = 0;
      int i = 0;
      boolean negation = false;
      String piece = null;
      StringBuffer sb = new StringBuffer();
      char c = 0;
      Pattern p = null;
      Matcher m = null;

      for (idx = 0; idx < array.length; idx ++)
      {
         // isolate negation (!) case
         if (array[idx].length() > 0 && array[idx].charAt(0) == '!')
         {
            piece = array[idx].substring(1);
            negation = true;
         }
         else
         {
            piece = array[idx];
            negation = false;
         }

         // build a regex out of the list entry; only * is converted into .*
         sb.setLength(0);
         for (i = 0; i < piece.length(); i ++)
         {
            c = piece.charAt(i);
            if (c != '*')
               sb.append(fixupRegexChar(c));
            else
               sb.append(".*");
         }
         regex = sb.toString();

         // compile regex as case-insensitive and apply it to the item
         p = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);
         m = p.matcher(item);
         if (!m.matches())
            continue;

         // this is a matching entry from the list; produce the result
         result = !negation;
         break;
      }

      return new logical(result);
   }
   
   /**
    * Removes a set of enclosing double quotes (first and last characters)
    * and converts any doubled up instances (2 instances sequentally) of
    * double quotes into a single instance of double quotes. This reverses
    * the reversable portion of the export format into a normal string as
    * Java would expect to see it.
    *
    * @param    export
    *           A string with the contents in export format as created by
    *           {@link #toStringExport}.
    *
    * @return   A valid Java string.
    */
   public static String fromExportString(String export)
   {
      // this will always work as long as the input string is valid
      String contents = export.substring(1, export.length() - 1);
      
      // remove any doubling up of double quote chars and convert to a single
      // instance
      return contents.replaceAll("\"\"", "\"");
   }

   /**
    * Converts a Progress 4GL compatible string to the equivalent Java source 
    * string, by stripping off enclosing quotes/string options and converting
    * the contents (including rewriting all escape sequences). Note that the
    * output format is designed to be written to a Java source file, which
    * requires a different approach than writing a string that can be used 
    * at runtime.
    * <p>
    * Escaped characters (using either the tilde or the backslash character)
    * and special formations will be converted to native representations and
    * any characters that need to be escaped will be so escaped.  In
    * particular, almost all usage of the backslash is compatible between
    * Progress and Java, so only those constructs that must be different
    * will be handled differently.
    * <p>
    * The following conversions will be made:
    * <p>
    * <pre>
    * ~b       to   '\b'   - backspace
    * ~t       to   '\t'   - horizontal tab
    * ~n       to   '\n'   - linefeed
    * ~f       to   '\f'   - formfeed
    * ~r       to   '\r'   - carriage return
    * ~&quot;       to   '\&quot;'   - double quote
    * ~&apos;       to   '\&apos;'   - single quote
    * ~\       to   '\\'   - backslash
    * ~~       to   '~'    - single tilde
    * ~{       to   '{'    - left curly brace
    * ~E       to   '\033' - the ASCII escape char
    * ~nnn     to   '\nnn' - octal escape sequence (nnn is a 3 digit octal number)
    * &apos;&apos;       to   '\&apos;'   - single quote (only when inside a
    *                                       string that was enclosed by single
    *                                       quotes)
    * &quot;&quot;       to   '\&quot;'   - double quote (only when inside a
    *                                       string that was enclosed by double
    *                                       quotes)
    * \~       to   '~'    - single tilde 
    * \\       to   '\\'   - backslash
    * \{       to   '{'    - left curly brace
    * \E       to   '\033' - the ASCII escape char
    * </pre>
    * <p>
    * Any other escape sequence that is unrecognized will result in the
    * tilde or backslash being dropped and the following character output
    * unchanged which is the same behavior as Progress.
    * <p>
    * The final result when output will be output inside enclosing double
    * quotes but there is no need for those quotes inside the AST form.    
    *
    * @param    progress
    *           The Progress compatible string as parsed from the source
    *           file.
    *
    * @return   A valid Java string that can be written as a string literal.
    */
   public static String progressToJavaString(String progress)
   throws IllegalArgumentException
   {  
      if (progress == null)
         throw new IllegalArgumentException("Invalid input: <null>.");
      
      // are we starting with ' or "
      char lead = progress.charAt(0);
      
      String contents = null;
      
      try
      {   
         // this will always work as long as the input string is valid
         contents = progress.substring(1, progress.lastIndexOf(lead));
      }
      
      catch (IndexOutOfBoundsException iob)
      {
         throw new IllegalArgumentException("Missing closing quote (" +
                                            lead + ").");
      }
      
      StringBuffer sb = new StringBuffer();
      
      int end = contents.length();
      
      // quick exit for the empty string
      if (end == 0)
      {
         return "";
      }
            
      // by now the enclosing quotes are gone, as are any string options
      // so we must convert the contents into the equivalent Java form
      for (int i = 0; i < end; i++) 
      {
         char current = contents.charAt(i);
         char next;
         
         switch (current)
         {
            case '\'':
            case '\"':
               // does the quote type match our enclosing quotes or not?
               if (current == lead)
               {
                  // this should only occur if we have "doubled up" quotes
                  // and we need to "consume" the extra character
                  i++;
                  next = readChar(contents, i);
                  
                  // check to ensure we aren't smoking something here
                  if (next != current)
                     throw new IllegalArgumentException("Malformed string.");
               }
               
               // escape the double quote as in Progress it safely stands 
               // alone
               if (current == '\"')
               {
                  sb.append('\\');
               }
               
               sb.append(current);
               break;
            case '~':
            case '\\':
               // there must always be a following character
               i++;
               next = readChar(contents, i);
               
               // handle the special cases that don't have a one-to-one
               // correspondence in Java
               if (next == 'E')
               {
                  // the special case for the ESC char
                  sb.append("\\033");
               }
               else
               {
                  // only certain chars need to be AND in fact other than the
                  // single quote (which we deliberately omit here) only 
                  // these MAY BE escaped in Java so drop the preceding
                  // escape char unless the following char is in this list
                  if (next == 'b'  || next == 't'  || next == 'n'  ||
                      next == 'f'  || next == 'r'  || next == '\"' || 
                      next == '\\' || (next >= '0' && next <= '7'))
                  {
                     // we must convert the leading tilde into the Java
                     // escape char because all of these must be escaped in
                     // Java
                     sb.append('\\');
                  }

                  // insert our following char (warning: this is a bit funky
                  // for the octal escapes because we don't write the whole
                  // thing at once, but the following chars will all trip
                  // the default processing and be passed through so this
                  // works even if it isn't as obvious as we would like)
                  sb.append(next);                  
               }               
               break;            
            default:
               // not an escaped char or special case
               sb.append(current);
         }
      }
      
      return sb.toString();
   }   
   
   /**
    * Return a string which contains the contents of the given string up to 
    * (but not including) the first <code>null</code> character (encoded as a
    * Progress escape sequence) and for that <code>null</code> character and 
    * all following characters the resulting string will contain spaces. 
    * <p>
    * The resulting length of the string (in terms of the number of
    * characters that the string will include at runtime) will NOT change,
    * although the length of the string representation (as a source code
    * representation stored in a Java string) may be reduced as the
    * <code>null</code> character's escape sequence (which would require 4
    * characters to represent as in <code>~000</code>) and any following
    * escaped sequences will be converted to a space which only consumes
    * a single character position.
    * <p>
    * The enclosing single or double quote characters are maintained (not
    * overwritten).
    * <p>
    * This processing is needed by the Progress preprocessor and other
    * conversion processing of string literals.    
    * <p>
    * This transformation is for Progress compatible string literals, NOT for
    * runtime strings.  If a runtime string conversion is needed, please use
    * {@link #javaSpacifyNull}.  Both forms of the <code>null</code>
    * character escape sequence are handled.
    * <p>
    * As an example, a Progress string such as:
    * <p>
    * <pre>
    * Input                               Output
    * ----------------------------------  ----------------------------------
    * "This has a ~000 (null) char."      "This has a               "      
    * 'This has a ~000 (null) char.'      'This has a               '
    * "~000"                              " "
    * '~000'                              ' '
    * "\000"                              " "
    * '\000'                              ' '
    * </pre>
    *
    * @param    data
    *           A Progress source code representation of a string literal
    *           which is the input string to be transformed.
    * @param    unixEsc
    *           <code>true</code> if the backslash character should be
    *           treated as an escape character.
    *
    * @return   The "space-ified" output string (still valid as a Progress
    *           string literal that could be output into source code).           
    */
   public static String progressSpacifyNull(String data, boolean unixEsc)
   {
      StringBuffer sb = new StringBuffer();
      
      // put the leading single or double quote char into the output buffer 
      char quote = data.charAt(0);
      sb.append(quote);
      
      // only scan up to the trailing quote char
      int end = data.length() - 1;
      
      boolean eat = false;
      
      // loop through the contents
      for (int i = 1; i < end; i++) 
      {
         char current = data.charAt(i);
         char next;
         
         if (current == '~' || (current == '\\' && unixEsc))
         {
            if (!eat)
            {
               // flow through mode
               
               // check for the null escape sequence
               String esc = ((i + 4) <= end) ? data.substring(i + 1, i + 4)
                                             : null;
               
               if (esc != null && esc.equals("000"))
               {
                  // the first null, shift into space-ify mode
                  eat = true;
                  i += 3;
                  sb.append(' ');
               }
               else
               {
                  // some other escape sequence, allow it to flow through
                  sb.append(current);
                  
                  // there must always be a following character, if it is
                  // a doubled escape char, we must handle it here to avoid 
                  // dropping back through this code since it is not the
                  // opening of a new escape sequence (since all following
                  // chars can be handled the same way we just go ahead
                  // no matter which one it is)
                  i++;
                  sb.append(readChar(data, i));
               }
            }
            else
            {
               // space-ify mode (eat the full escape sequence here)
               
               // output the replacement (always 1 char)
               sb.append(' ');
               
               // there must always be at least 1 following character
               i++;
               next = readChar(data, i);
               
               if (next == 'b'  || next == 't'  || next == 'n'  ||
                   next == 'f'  || next == 'r'  || next == '\"' || 
                   next == '\\' || next == '~'  || next == 'E')
               {
                  // these are only 1 char in size, so we are done
               }
               else if (next >= '0' && next <= '7')
               {
                  // eat the next 2 chars too because this is an octal
                  // escape sequence
                  i += 2;
               }               
            }
         }
         else
         {
            sb.append(eat ? ' ' : current);
         }
      }
      
      // add the trailing (close) quote char
      sb.append(data.charAt(end));
      
      return sb.toString();      
   }
   
   /**
    * Return a string which contains the contents of the given string up to 
    * (but not including) the first <code>null</code> character (ASCII 0x00)
    * and for that <code>null</code> character and all following characters
    * the resulting string will contain spaces. 
    * <p>
    * The resulting length of the output string will be the same as the
    * length of the input string.
    * <p>
    * This processing is needed by some operators and/or functionality that
    * process <code>character</code> data.
    * <p>
    * This transformation is for runtime strings, NOT for Progress compatible
    * string literals.  If a source code conversion is needed, please use
    * {@link #progressSpacifyNull}.
    * <p>
    * As an example, a string such as:
    * <p>
    * <pre>
    * Input                               Output
    * ----------------------------------  ----------------------------------
    * "This has a \0 (null) char."        "This has a               "      
    * "\0"                                " "
    * </pre>
    *
    * @param    data
    *           The input string (not valid for encoding in a source file).
    *
    * @return   The "space-ified" output string (still not valid for
    *           encoding in a source file).           
    */
   public static String javaSpacifyNull(String data)
   {
      StringBuffer sb = null;
      
      int idx = data.indexOf('\0');
      
      if (idx != -1)
      {
         sb = new StringBuffer(data.substring(0, idx));
         StringHelper.repeatChar(' ', (data.length() - idx), sb);
      }
      
      return sb == null ? data : sb.toString();
   }
   
   /**
    * Return a string which contains the contents of the given string up to 
    * (but not including) the first <code>null</code> character (ASCII 0x00),
    * the resulting string is truncated at the first <code>null</code>.
    * <p>
    * As an example, a string such as:
    * <p>
    * <pre>
    * Input                               Output
    * ----------------------------------  ----------------------------------
    * "This has a \0 (null) char."        "This has a "      
    * "\0"                                ""
    * </pre>
    *
    * @param    data
    *           The input string (not valid for encoding in a source file).
    *
    * @return   The truncated output string (still not valid for
    *           encoding in a source file).           
    */
   public static String javaTruncateNull(String data)
   {
      StringBuffer sb = null;
      
      int idx = data.indexOf('\0');
      
      if (idx != -1)
      {
         sb = new StringBuffer(data.substring(0, idx));
      }
      
      return sb == null ? data : sb.toString();
   }
   
   /**
    * Returns the numeric value of a character expression which must be
    * only 1 character in length.  This method will probably generate
    * incorrect results in a DBCS environment since it will use UNICODE
    * as the basis for conversion.  If this returned value is different than
    * would be returned under Progress AND if that Progress-specific value 
    * is used in comparison with this returned value, then a difference in
    * logic will be seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    *
    * @param    c
    *           The character expression to convert (must be 1 character in
    *           length).
    *
    * @return    The numeric value of that character which should match up
    *            with the Progress equivalents in the ASCII range.
    */
   public static integer asc(character c)
   {
      if (c.unknown)
      {
         return new integer();
      }
      
      return asc(c.value);
   }
      
   /**
    * Returns the numeric value of a string which must be only 1 character
    * in length.  This method will probably generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for conversion.
    * If this returned value is different than would be returned under
    * Progress AND if that Progress-specific value is used in comparison with
    * this returned value, then a difference in logic will be seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    *
    * @param    str
    *           The string to convert (must be 1 character in length).
    *
    * @return   The numeric value of that character which should match up
    *           with the Progress equivalents in the ASCII range.
    */
   public static integer asc(String str)
   {
      int i = -1;
      
      if (str.length() == 1)
      {
         i = Utils.toInt(str.charAt(0), override);
      }
      
      return new integer(i);
   }
      
   /**
    * Returns the character value of a numeric expression which must be
    * between 1 and 65534 (inclusive).  This method will probably generate
    * incorrect results in a DBCS environment since it will use UNICODE
    * as the basis for conversion.  If this returned value is different than
    * would be returned under Progress AND if that Progress-specific value 
    * is used in comparison with this returned value, then a difference in
    * logic will be seen.  The result should match up with the Progress
    * equivalents in the ASCII range.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    *
    * @param    i
    *           The integer to convert.
    *
    * @return   The associated character or a character representing the
    *           empty string (0 length) when the input is out of range or
    *           <code>unknown value</code> if the input is
    *           <code>unknown value</code>.
    */
   public static character chr(integer i)
   {
      if (i.isUnknown())
      {
         return new character();
      }
      
      return chr(i.intValue());
   }
      
   /**
    * Returns the character value of a numeric expression which must be
    * between 1 and 65534 (inclusive).  This method will probably generate
    * incorrect results in a DBCS environment since it will use UNICODE
    * as the basis for conversion.  If this returned value is different than
    * would be returned under Progress AND if that Progress-specific value 
    * is used in comparison with this returned value, then a difference in
    * logic will be seen.  The result should match up with the Progress
    * equivalents in the ASCII range.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    *
    * @param    i
    *           The integer to convert.
    *
    * @return   The associated character or a character representing the
    *           empty string (0 length) when the input is out of range.
    */
   public static character chr(int i)
   {
      if (i < 1 || i > 65534)
      {
         return new character("");
      }
      
      char ch = Utils.toChar(i, override);
      
      return (ch == 0xFFFF) ? new character("") : new character(ch);
   }
      
   /**
    * Compares two character values with a runtime-specified strength and
    * operator.  This method will probably generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for conversion.
    * The result should match up with the Progress equivalents in the ASCII
    * range.
    * <p>
    * This method does not handle the case of user-specified collation tables
    * as is possible in Progress.
    * <p>
    * In addition, the case-sensitive and case-insensitive strength options
    * are likely to differ in some results since the current implementation
    * really just implements a direct numeric comparison based on each 
    * UNICODE character's value (though case-insensitive support is
    * provided but it just converts everything to the same case before
    * implementing the 'raw' lexicographic comparison.
    *
    * @param    op1
    *           The left operand.
    * @param    operator
    *           One of {@link character#LT}, {@link character#LE},
    *           {@link character#GT}, {@link character#GE},
    *           {@link character#EQ} or {@link character#NE}.
    * @param    op2
    *           The right operand.
    * @param    strength
    *           One of {@link character#RAW}, {@link character#CASE_SENSITIVE}
    *           or {@link character#CASE_INSENSITIVE}.
    *
    * @return   The result of the condition evaluation. 
    */   
   public static logical compare(character op1,
                                 int       operator,
                                 character op2,
                                 int       strength)
   {
      if (operator != LT && operator != LE && operator != GT &&
          operator != GE && operator != EQ && operator != NE) 
      {
         // unrecognized operator
         return new logical();
      }
      
      // unknown value logic
      if (op1.unknown && op2.unknown)
      {
         // both are unknown
         if (operator == LE || operator == GE || operator == EQ) 
         {
            return new logical(true);
         }
         else
         {
            return new logical(false);
         }
      }
      else
      {
         // both are NOT unknown BUT...
         if (op1.unknown || op2.unknown)
         {
            // one or the other is unknown
            if (operator == NE)
            {
               return new logical(true);
            }
            else
            {
               return new logical(false);
            }
         }
      }
      
      int result;
      
      if (strength == RAW || strength == CASE_SENSITIVE)
      {
         // RAW and CASE_SENSITIVE are the same right now
         result = op1.value.compareTo(op2.value);
      }
      else if (strength == CASE_INSENSITIVE) 
      {
         result = op1.value.compareToIgnoreCase(op2.value);
      }
      else
      {
         // unrecognized strength
         return new logical();
      }
      
      boolean conclude = false;
      
      switch (operator)
      {
         case LT:
            conclude = (result < 0);
            break;
         case LE:
            conclude = (result <= 0);
            break;
         case GT:
            conclude = (result > 0);
            break;
         case GE:
            conclude = (result >= 0);
            break;
         case EQ:
            conclude = (result == 0);
            break;
         case NE:
            conclude = (result != 0);
            break;
      }
      
      return new logical(conclude);
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i,
                                 character  list,
                                 character  delimit)
   throws ErrorConditionException
   {
      if (i.isUnknown() || list.unknown || delimit.unknown)
      {
         return new character();
      }
      
      return entry(i.intValue(), list.value, safeDelimiter(delimit.value));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double i, character list, character delimit)
   throws ErrorConditionException
   {
      if (list.unknown || delimit.unknown)
      {
         return new character();
      }
      
      return entry((int) i, list.value, safeDelimiter(delimit.value));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i, String list, character delimit)
   throws ErrorConditionException
   {
      if (i.isUnknown() || delimit.unknown)
      {
         return new character();
      }
      
      return entry(i.intValue(), list, safeDelimiter(delimit.value));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i,
                                 character  list,
                                 String     delimit)
   throws ErrorConditionException
   {
      if (i.isUnknown() || list.unknown)
      {
         return new character();
      }
      
      return entry(i.intValue(), list.value, safeDelimiter(delimit));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double i, character list, String delimit)
   throws ErrorConditionException
   {
      if (list.isUnknown())
      {
         return new character();
      }
      
      return entry((int) i, list.value, safeDelimiter(delimit));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i, String list, String delimit)
   throws ErrorConditionException
   {
      if (i.isUnknown())
      {
         return new character();
      }
      
      return entry(i.intValue(), list, safeDelimiter(delimit));
   }
   
   /**
    * Returns the specified 1-based indexed comma-delimited entry in the
    * passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i, character list)
   throws ErrorConditionException
   {
      if (i.isUnknown() || list.unknown)
      {
         return new character();
      }
      
      return entry(i.intValue(), list.value, ',');
   }
   
   /**
    * Returns the specified 1-based indexed comma-delimited entry in the
    * passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(NumberType i, String list)
   throws ErrorConditionException
   {
      if (i.isUnknown())
      {
         return new character();
      }
      
      return entry(i.intValue(), list, ',');
   }
   
   /**
    * Returns the specified 1-based indexed comma-delimited entry in the
    * passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double i, character list)
   throws ErrorConditionException
   {
      if (list.isUnknown())
      {
         return new character();
      }
      
      return entry((int) i, list.value, ',');
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double i, String list, String delimit)
   throws ErrorConditionException
   {
      return entry((int) i, list, safeDelimiter(delimit));
   }
   
   /**
    * Returns the specified 1-based indexed entry in the passed list.  
    *
    * @param    ii
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double ii, String list, char delimit)
   throws ErrorConditionException
   {
      // create the regular expression (with proper escapes)
      String regex = fixupRegexChar(delimit);      
      
      // split up the list (make sure all possible entries are included,
      // even empty trailing entries --> use -1 as the limit to force this) 
      String[] array = list.split(regex, -1);
      
      // convert 1 based index to 0 based
      int i   = (int) ii;
      int idx = i - 1;
      
      if (idx < 0 || idx >= array.length)
      {
         genOutsideRangeError(i, list);
         
         // this is the safest thing to return (it should be undone anyway)
         return new character("");
      }
      
      return new character(array[idx]);
   }
   
   /**
    * Returns the specified 1-based indexed comma-delimited entry in the
    * passed list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The entry in the list or the <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static character entry(double i, String list)
   throws ErrorConditionException
   {
      return entry((int) i, list, ',');
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, character list, character delim)
   {
      if (c.unknown || list.unknown || delim.unknown)
      {
         return new integer();
      }
      
      boolean cs = (c.caseSens || list.caseSens);
      
      return lookup(c.value, list.value, safeDelimiter(delim.value), cs);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, String list, character delim)
   {
      if (c.unknown || delim.unknown)
      {
         return new integer();
      }
      
      return lookup(c.value, list, safeDelimiter(delim.value), c.caseSens);
   }
   
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, character list, character delim)
   {
      if (list.unknown || delim.unknown)
      {
         return new integer();
      }
      
      return lookup(c, list.value, safeDelimiter(delim.value), list.caseSens);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, character list, String delim)
   {
      if (c.unknown || list.unknown)
      {
         return new integer();
      }
      
      boolean cs = (c.caseSens || list.caseSens);
      
      return lookup(c.value, list.value, safeDelimiter(delim), cs);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * comma-delimited list.  The search will be case-sensitive if either
    * argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, character list)
   {
      if (c.unknown || list.unknown)
      {
         return new integer();
      }
      
      boolean cs = (c.caseSens || list.caseSens);
      
      return lookup(c.value, list.value, ',', cs);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * comma-delimited list.  The search will be case-sensitive if either
    * argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, String list)
   {
      if (c.unknown)
      {
         return new integer();
      }
      
      return lookup(c.value, list, ',', c.caseSens);
   }
   
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * comma-delimited list.  The search will be case-sensitive if either
    * argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, character list)
   {
      if (list.unknown)
      {
         return new integer();
      }
      
      return lookup(c, list.value, ',', list.caseSens);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter.  The search will be done
    * case-insensitively.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found.
    */
   public static integer lookup(String c, String list, char delim)
   {
      return lookup(c, list, delim, false);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter.  
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    * @param    cs
    *           <code>true</code> specifies the match is to be done with
    *           case-sensitivity.
    *
    * @return   The 1-based index into the list or 0 if no match is found.
    */
   public static integer lookup(String c, String list, char delim, boolean cs)
   {
      // create the regular expression (with proper escapes)
      String regex = fixupRegexChar(delim);      
      
      // split up the list (make sure all possible entries are included,
      // even empty trailing entries --> use -1 as the limit to force this) 
      String[] array = list.split(regex, -1);
      
      // only use up to the first instance of the delimiter in the search 
      // pattern
      int    idx    = c.indexOf(delim);
      String search = idx < 0 ? c : c.substring(0, idx); 
      
      // default return value
      int index  = 0;
      
      // do our search
      for (int i = 0; i < array.length; i++)
      {
         if (cs ? array[i].equals(search) : array[i].equalsIgnoreCase(search))
         {
            // match found!  convert from 0-based to 1-based
            index = i + 1;
            break;
         }
      }
      
      return new integer(index);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * comma-delimited list.  The search will be done case-insensitively.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, String list)
   {
      return lookup(c, list, ',', false);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, String list, String delim)
   {
      return lookup(c, list, safeDelimiter(delim), false);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, character list, String delim)
   {
      if (list.unknown)
      {
         return new integer();
      }
      
      return lookup(c, list.value, safeDelimiter(delim), list.caseSens);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(character c, String list, String delim)
   {
      if (c.unknown)
      {
         return new integer();
      }
      
      return lookup(c.value, list, safeDelimiter(delim), c.caseSens);
   }
   
   /**
    * Returns the 1-based index of a specified entry in the passed 
    * list with a user-specified delimiter. The search will be case-sensitive
    * if either argument is case-sensitive.
    *
    * @param    c
    *           The entry to find in the list.
    * @param    list
    *           The delimited list of entries.
    * @param    delim
    *           The delimiter character used to parse the list into entries.
    *
    * @return   The 1-based index into the list or 0 if no match is found or
    *           the <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public static integer lookup(String c, String list, character delim)
   {
      if (delim.unknown)
      {
         return new integer();
      }
      
      return lookup(c, list, safeDelimiter(delim.value), false);
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list or -1 if the list is the
    *           <code>unknown value</code>.
    */
   public static int numEntriesOf(character list, character delimit)
   {
      if (list.unknown || delimit.unknown)
      {
         return -1;
      }
      
      return numEntriesOf(list.value, safeDelimiter(delimit.value));
   }
   
   /**
    * Returns the number of comma-delimited entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The number of entries in the list or -1 if the list is the
    *           <code>unknown value</code>.
    */
   public static int numEntriesOf(character list)
   {
      if (list.unknown)
      {
         return -1;
      }
      
      return numEntriesOf(list.value, ',');
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list.
    */
   public static int numEntriesOf(String list, char delimit)
   {
      if (list.length() == 0)
      {
         return 0;
      }
      
      // create the regular expression (with proper escapes)
      String regex = fixupRegexChar(delimit);      
      
      // split up the list (make sure all possible entries are included,
      // even empty trailing entries --> use -1 as the limit to force this) 
      String[] array = list.split(regex, -1);
      
      return array.length;
   }
   
   /**
    * Returns the number of comma-delimited entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The number of entries in the list.
    */
   public static int numEntriesOf(String list)
   {
      return numEntriesOf(list, ',');
   }
   
   /**
    * Returns the number of comma-delimited entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The number of entries in the list or the
    *           <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static integer numEntries(character list)
   {
      if (list.unknown)
      {
         return new integer();
      }
      
      return new integer(numEntriesOf(list.value));
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list or the
    *           <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static integer numEntries(character list, String delimit)
   {
      if (list.unknown)
      {
         return new integer();
      }
      
      return new integer(numEntriesOf(list.value, safeDelimiter(delimit)));
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list or the
    *           <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static integer numEntries(String list, character delimit)
   {
      if (delimit.unknown)
      {
         return new integer();
      }
      
      return new integer(numEntriesOf(list, safeDelimiter(delimit.value)));
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list or the
    *           <code>unknown value</code> if
    *           any of the inputs are <code>unknown value</code>.
    */
   public static integer numEntries(character list, character delimit)
   {
      if (list.unknown || delimit.unknown)
      {
         return new integer();
      }
      
      return new integer(numEntriesOf(list.value,
                                      safeDelimiter(delimit.value)));
   }
   
   /**
    * Returns the number of entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    * @param    delimit
    *           The delimiter character.
    *
    * @return   The number of entries in the list.
    */
   public static integer numEntries(String list, char delimit)
   {
      return new integer(numEntriesOf(list, delimit));
   }
   
   /**
    * Returns the number of comma-delimited entries in the passed list.  
    *
    * @param    list
    *           The delimited list of entries.
    *
    * @return   The number of entries in the list.
    */
   public static integer numEntries(String list)
   {
      return new integer(numEntriesOf(list));
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    c
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(character c, NumberType num)
   {
      if (c.unknown || num.isUnknown())
      {
         return new character();
      }
      
      return fill(c.value, num.intValue());
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    c
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(character c, int num)
   {
      if (c.isUnknown())
      {
         return new character();
      }
      
      return fill(c.value, num);
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    c
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(character c, double num)
   {
      if (c.isUnknown())
      {
         return new character();
      }
      
      return fill(c.value, (int) num);
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    str
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(String str, NumberType num)
   {
      if (num.isUnknown())
      {
         return new character();
      }
      
      return fill(str, num.intValue());
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    str
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(String str, double num)
   {
      return fill(str, (int) num);
   }
   
   /**
    * Return a character instance that is comprised of <code>num</code>
    * concatinated copies of a given string. 
    *
    * @param    str
    *           The string to be repeated.
    * @param    num
    *           The number of times to repeat.
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code> or an empty string if 
    *           <code>num</code> < 1.
    */
   public static character fill(String str, int num)
   {
      if (num < 1)
         return new character("");
      
      StringBuffer sb = new StringBuffer();
      
      for (int i = 0; i < num; i++)
      {
         sb.append(str);
      }
      
      return new character(sb.toString());
   }
   
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character  src,
                                 character  target,
                                 NumberType i)
   {
      if (src.unknown || target.unknown || i.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src.value,
                     target.value,
                     i.doubleValue(),
                     src.isCaseSensitive() || target.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(String     src,
                                 character  target,
                                 NumberType i)
   {
      if (target.isUnknown() || i.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src,
                     target.value,
                     i.doubleValue(),
                     target.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character  src,
                                 String     target,
                                 NumberType i)
   {
      if (src.isUnknown() || i.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src.value,
                     target,
                     i.doubleValue(),
                     src.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(String     src,
                                 String     target,
                                 NumberType i)
   {
      if (i.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src, target, i.doubleValue(), false);
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character  src,
                                 character  target,
                                 double     i)
   {
      if (src.unknown || target.unknown)
      {
         return new integer();
      }
      
      return indexOf(src.value,
                     target.value,
                     i,
                     src.isCaseSensitive() || target.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(String     src,
                                 character  target,
                                 double     i)
   {
      if (target.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src, target.value, i, target.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character  src,
                                 String     target,
                                 double     i)
   {
      if (src.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src.value, target, i, src.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character src,
                                 character target)
   {
      if (src.unknown || target.unknown)
      {
         return new integer();
      }
      
      return indexOf(src.value,
                     target.value,
                     1,
                     src.isCaseSensitive() || target.isCaseSensitive());
   }
   
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(String    src,
                                 character target)
   {
      if (target.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src, target.value, 1, target.isCaseSensitive());
   }
      
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer indexOf(character src,
                                 String    target)
   {
      if (src.isUnknown())
      {
         return new integer();
      }
      
      return indexOf(src.value, target, 1, src.isCaseSensitive());
   }
   
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    ii
    *           The (1-based) starting index from which to search.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found.
    */
   public static integer indexOf(String src, String target, double ii)
   {
      return indexOf(src, target, ii, false);
   }
   
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found.
    */
   public static integer indexOf(String src, String target)
   {
      return indexOf(src, target, 1, false);
   }
   
   /**
    * Returns the 1-based index of one character expression within another
    * given character expression based on a known starting point for the
    * search.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    ii
    *           The (1-based) starting index from which to search.
    * @param    caseSensitive
    *           <code>true</code> if case must be considered in the search;
    *           <code>false</code> if the search is case insensitive.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found.
    */
   public static integer indexOf(String src,
                                 String target,
                                 double ii,
                                 boolean caseSensitive)
   {
      int i = (int) ii;
      
      if (src.length() == 0 || target.length() == 0 || i < 1)
      {
         return new integer(0);
      }
      
      if (!caseSensitive)
      {
         src = src.toLowerCase();
         target = target.toLowerCase();
      }
      
      return new integer(src.indexOf(target, i - 1) + 1);
   }
   
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character  src,
                                     character  target,
                                     NumberType i)
   {
      if (src.unknown || target.unknown || i.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target.value, i.intValue());
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(String     src,
                                     character  target,
                                     NumberType i)
   {
      if (target.isUnknown() || i.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src, target.value, i.intValue());
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character  src,
                                     String     target,
                                     NumberType i)
   {
      if (src.isUnknown() || i.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target, i.intValue());
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(String     src,
                                     String     target,
                                     NumberType i)
   {
      if (i.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src, target, i.intValue());
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character src,
                                     character target,
                                     double    i)
   {
      if (src.isUnknown() || target.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target.value, (int) i);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(String    src,
                                     character target,
                                     double    i)
   {
      if (target.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src, target.value, (int) i);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    i
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character src,
                                     String    target,
                                     double    i)
   {
      if (src.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target, (int) i);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning).
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character src, character target)
   {
      if (src.unknown || target.unknown)
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target.value);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning).
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(String src, character target)
   {
      if (target.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src, target.value);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning).
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found or <code>unknown value</code>
    *           if any input is <code>unknown value</code>.
    */
   public static integer lastIndexOf(character src, String target)
   {
      if (src.isUnknown())
      {
         return new integer();
      }
      
      return lastIndexOf(src.value, target);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning) starting at a specified
    * starting index.
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    * @param    ii
    *           The starting index to search from.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found.
    */
   public static integer lastIndexOf(String src, String target, double ii)
   {
      int i = (int) ii;
      
      if (src.length() == 0 || target.length() == 0 || i < 1)
      {
         return new integer(0);
      }
      
      return new integer(src.lastIndexOf(target, i - 1) + 1);
   }
      
   /**
    * Returns the last 1-based index of one character expression within
    * another given character expression (the search starts at the end
    * of the string and moves toward the beginning).
    *
    * @param    src
    *           The character expression to search in.
    * @param    target
    *           The character expression to search for.
    *
    * @return   The 1-based index in the source at which the target is found
    *           or 0 if the target is not found.
    */
   public static integer lastIndexOf(String src, String target)
   {
      return lastIndexOf(src, target, src.length() + 1);
   }
      
   /**
    * Returns the length of the given <code>character</code> instance in
    * characters.
    *
    * @param    c
    *           The instance to test.
    *
    * @return   The number of characters in the given instance or -1 if the
    *           instance is <code>unknown value</code>.
    */
   public static int lengthOf(character c)
   {
      if (c.unknown)
         return -1;
      
      return c.value.length();
   }
      
   /**
    * Returns the length of the given <code>character</code> instance in
    * bytes.
    *
    * @param    c
    *           The instance to test.
    *
    * @return   The number of bytes in the given instance or -1 if the
    *           instance is <code>unknown value</code>.
    */
   public static int byteLengthOf(character c)
   {
      if (c.unknown)
         return -1;
      
      return c.value.getBytes().length;
   }
      
   /**
    * Returns the length of the given <code>character</code> instance in
    * characters.
    *
    * @param    c
    *           The instance to test.
    *
    * @return   The number of characters in the given instance
    *           <code>unknown value</code> if the instance is
    *           <code>unknown value</code>.
    */
   public static integer length(character c)
   {
      if (c.unknown)
         return new integer();
      
      return new integer(c.value.length());
   }
      
   /**
    * Returns the length of the given string instance in characters.
    *
    * @param    c
    *           The instance to test.
    *
    * @return   The number of characters in the given instance.
    */
   public static integer length(String c)
   {
      return new integer(c.length());
   }
      
   /**
    * Returns the length of the given <code>character</code> instance in
    * bytes.
    *
    * @param    c
    *           The instance to test.
    *
    * @return   The number of bytes in the given instance or
    *           <code>unknown value</code> if the instance is
    *           <code>unknown value</code>.
    */
   public static integer byteLength(character c)
   {
      if (c.unknown)
         return new integer();
      
      return new integer(c.value.getBytes().length);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.  
    * <p>
    * The search and the returned instance are case-sensitive if any of the
    * inputs are case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code>.
    */
   
   /**
    * Quoter function implementation with <code>character</code> specifics. 
    * This puts the <code>character</code> value between double quotes if the value is not unknown or replace it with '?' if the value is unknown.
    * If the character variable contains double quotes inside, it will be doubled(another double quote will be added next to it(Progress behaviour).
    * @param charVar The <code>character</code> variable to operate upon.
    * @return The resulting character value of the function.
    */
	public static character quoter(character charVar)
	{
      character result = null;
      if (!charVar.isUnknown())
      {
         String var = charVar.value;
         String in = "" + '"';
         String out = "" + '"' + '"';
         String var2 = var.replaceAll(in, out);
         result = concat('"', var2, '"');
      } else
      {
         result = new character("?");
      }
      return result;
   }
	/**
	 * General quoter function implementation.
	 * This puts the <code>character</code> value of the variable between double quotes if the value is not unknown or replace it with '?' if the value is unknown. 
	 * @param var the <code>BaseDataType</code> variable to operate upon 
	 * @return The resulting character value of the function.
	 */
   public static character quoter(BaseDataType var)
   {
      character result = null;
      if (!var.isUnknown())
      {
         result = concat('"', var, '"');
      } else
      {
         result = new character("?");
      }
      return result;
   }
   
   public static character replaceAll(character src,
                                      character from,
                                      character to)
   {
      if (src.unknown || from.unknown || to.unknown)
      {
         return new character();
      }
      
      boolean cs = (src.caseSens || from.caseSens || to.caseSens);
      
      return replaceAll(src.value, from.value, to.value, cs);
   }
      
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.  
    * <p>
    * The search and the returned instance are case-sensitive if any of the
    * inputs are case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code>.
    */
   public static character replaceAll(character src,
                                      character from,
                                      String    to)
   {
      if (src.unknown || from.unknown)
      {
         return new character();
      }
      
      boolean cs = (src.caseSens || from.caseSens);
      
      return replaceAll(src.value, from.value, to, cs);
   }
      
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.  
    * <p>
    * The search and the returned instance are case-sensitive if any of the
    * inputs are case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code>.
    */
   public static character replaceAll(character src,
                                      String    from,
                                      character to)
   {
      if (src.unknown || to.unknown)
      {
         return new character();
      }
      
      boolean cs = (src.caseSens || to.caseSens);
      
      return replaceAll(src.value, from, to.value, cs);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.  
    * <p>
    * The search and the returned instance are case-sensitive if any of the
    * inputs are case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if any
    *           input is <code>unknown value</code>.
    */
   public static character replaceAll(String    src,
                                      character from,
                                      character to)
   {
      if (from.unknown || to.unknown)
      {
         return new character();
      }
      
      boolean cs = (from.caseSens || to.caseSens);
      
      return replaceAll(src, from.value, to.value, cs);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.
    * <p>
    * The search and the returned instance are case-sensitive if the
    * <code>src</code> instance is case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if the
    *           <code>src</code> string is <code>unknown value</code>.
    */
   public static character replaceAll(character src, String from, String to)
   {
      if (src.unknown)
      {
         return new character();
      }
      
      return replaceAll(src.value, from, to, src.caseSens);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.
    * <p>
    * The search and the returned instance are case-sensitive if the
    * <code>from</code> instance is case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if the
    *           <code>from</code> string is <code>unknown value</code>.
    */
   public static character replaceAll(String src, character from, String to)
   {
      if (from.unknown)
      {
         return new character();
      }
      
      return replaceAll(src, from.value, to, from.caseSens);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.
    * <p>
    * The search and the returned instance are case-sensitive if the
    * <code>to</code> instance is case-sensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if the
    *           <code>to</code> string is <code>unknown value</code>.
    */
   public static character replaceAll(String src, String from, character to)
   {
      if (to.unknown)
      {
         return new character();
      }
      
      return replaceAll(src, from, to.value, to.caseSens);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.
    * <p>
    * The search and the returned instance are always case-insensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    *
    * @return   The resulting string or <code>unknown value</code> if the
    *           <code>src</code> string is <code>unknown value</code>.
    */
   public static character replaceAll(String src, String from, String to)
   {
      return replaceAll(src, from, to, false);
   }
   
   /**
    * Searches for all matches to the <code>from</code> string in the
    * <code>src</code> string and replaces these with the <code>to</code>
    * string.
    *
    * @param    src
    *           The string to operate upon.
    * @param    from
    *           The substring to search for. 
    * @param    to
    *           The substring to replace when a match is found. 
    * @param    cs
    *           Determines if a case-sensitive match should be performed AND
    *           the case-sensitivity of the returned instance.
    *
    * @return   The resulting string.
    */
   public static character replaceAll(String  src,
                                      String  from,
                                      String  to,
                                      boolean cs)
   {
      StringBuffer sb = new StringBuffer();
      
      int len  = src.length();
      int flen = from.length(); 
      
      for (int i = 0; i < len; i++)
      {
         if (src.regionMatches(!cs, i, from, 0, flen)) 
         {
            // make a replacement
            sb.append(to);
            
            // move past the end of the replaced text (take into account that
            // as the loop iterates, i will be incremented again)
            i += (flen - 1);
         }
         else
         {
            // no match, put the char into the output
            sb.append(src.charAt(i));
         }
      }
      
      return new character(sb.toString(), cs);
   }   
      
   /**
    * Perform positional replacement of '&amp;n' specifiers in the format string
    * <code>fmt</code>, using the array of <code>character</code> and/or
    * <code>String</code> objects as the list of replacements.  The 1-based
    * index position of the '&amp;n' specifiers (where n and range from 1 to 
    * 9) is converted to a 0-based index into the <code>replacements</code>
    * array.  Instances of '&amp;&amp;' yield a single '&amp;' in the result.
    * Any specifiers that do not correspond to an object in the array are
    * replaced with an empty string. 
    *
    * @param    fmt
    *           The string specifying the text and format specifiers. 
    * @param    replacements
    *           The list of replacements.
    *
    * @return   The resulting string or <code>unknown value</code> if the
    *           <code>fmt</code> string is <code>unknown value</code>.
    */
   public static character substitute(character fmt, Object[] replacements)
   throws ErrorConditionException
   {
      if (fmt.unknown)
         return new character();
      
      return substitute(fmt.value, replacements);
   }
   
   /**
    * Perform positional replacement of '&amp;n' specifiers in the format string
    * <code>fmt</code>, using the array of <code>character</code> and/or
    * <code>String</code> objects as the list of replacements.  The 1-based
    * index position of the '&amp;n' specifiers (where n and range from 1 to 
    * 9) is converted to a 0-based index into the <code>replacements</code>
    * array.  Instances of '&amp;&amp;' yield a single '&amp;' in the result.
    * Any specifiers that do not correspond to an object in the array are
    * replaced with an empty string. 
    *
    * @param    fmt
    *           The string specifying the text and format specifiers. 
    * @param    replacements
    *           The list of replacements.
    *
    * @return   The resulting string.
    */
   public static character substitute(String fmt, Object[] replacements)
   throws ErrorConditionException
   {
      StringBuffer sb = new StringBuffer();
      
      int len  = fmt.length();
      
      for (int i = 0; i < len; i++)
      {
         char c = fmt.charAt(i);
         
         if (c == '&') 
         {
            // ignore this char and look at the next one
            i++;
            c = fmt.charAt(i);
            
            if (c >= '0' && c <= '9')
            {
               // convert this character into a 0-based index into the array
               int idx = Character.digit(c, 10) - 1;
               
               // check if this is a valid replacement index (note that 0 is
               // always ignored/invalid), if not the result is an empty
               // string since we don't make any replacement in that case
               if (idx >= 0 && idx < replacements.length) 
               {
                  Object obj = replacements[idx];
                  
                  if (obj != null)
                  {
                     // make a replacement if it is an object we recognize
                     if (obj instanceof String)
                     {
                        sb.append((String) obj);   
                     }
                     else if (obj instanceof character)
                     {
                        character ch = (character) obj;
                        sb.append(ch.unknown ? "?" : ch.value);
                     }
                     else if (obj instanceof BaseDataType)
                     {
                        BaseDataType bdt = (BaseDataType) obj;
                        sb.append(bdt.toStringExport());
                     }
                  }
               }
            }            
            else if (c == '&')
            {
               sb.append(c);
            }
            else
            {
               String spec = "Invalid character \'%c\' following \'&\'" +
                             " in SUBSTITUTE";
               ErrorManager.recordOrThrowError(2920, String.format(spec, c));
               
               // this is the safest thing to return (it should be undone anyway)
               return new character("");
            }
         }
         else
         {
            // no match, put the char into the output
            sb.append(c);
         }
      }
      
      return new character(sb.toString());
   }   
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     NumberType len,
                                     String     type)
   {
      if (src.unknown || pos.isUnknown() || len.isUnknown())
         return new character();
      
      return substring(src.value,
                       pos.intValue(),
                       len.intValue(),
                       type,
                       src.caseSens);
   }
         
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     NumberType len,
                                     character  type)
   {
      if (src.unknown || pos.isUnknown() || len.isUnknown() || type.unknown)
         return new character();
      
      return substring(src.value,
                       pos.intValue(),
                       len.intValue(),
                       type.value,
                       src.caseSens);
   }
   
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     int        pos,
                                     NumberType len,
                                     String     type)
   {
      if (src.unknown || len.isUnknown())
         return new character();
      
      return substring(src.value,
                       pos,
                       len.intValue(),
                       type,
                       src.caseSens);
   }
   
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     int        pos,
                                     NumberType len,
                                     character  type)
   {
      if (src.unknown || len.isUnknown() || type.unknown)
         return new character();
      
      return substring(src.value,
                       pos,
                       len.intValue(),
                       type.value,
                       src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     int        len,
                                     String     type)
   {
      if (src.unknown || pos.isUnknown())
         return new character();
      
      return substring(src.value,
                       pos.intValue(),
                       len,
                       type,
                       src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     int        len,
                                     character  type)
   {
      if (src.unknown || pos.isUnknown() || type.unknown)
         return new character();
      
      return substring(src.value,
                       pos.intValue(),
                       len,
                       type.value,
                       src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if the
    *           <code>src</code> is <code>unknown</code>.
    */
   public static character substring(character src,
                                     int       pos,
                                     int       len,
                                     String    type)
   {
      if (src.unknown)
         return new character();
      
      return substring(src.value, pos, len, type, src.caseSens);
   }
   
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring or <code>unknown value</code> if the
    *           <code>src</code> is <code>unknown</code>.
    */
   public static character substring(character src,
                                     int       pos,
                                     int       len,
                                     character type)
   {
      if (src.unknown || type.unknown)
         return new character();
      
      return substring(src.value, pos, len, type.value, src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    * <p>
    * The returned instance is case-insensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    *
    * @return   The resulting substring.
    */
   public static character substring(String src,
                                     int    pos,
                                     int    len,
                                     String type)
   {
      return substring(src, pos, len, type, false);
   }
   
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     NumberType len)
   {
      if (src.unknown || pos.isUnknown() || len.isUnknown())
         return new character();
      
      return substring(src.value,
                       pos.intValue(),
                       len.intValue(),
                       null,
                       src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     int        pos,
                                     NumberType len)
   {
      if (src.unknown || len.isUnknown())
         return new character();
      
      return substring(src.value, pos, len.intValue(), null, src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character  src,
                                     NumberType pos,
                                     int        len)
   {
      if (src.unknown || pos.isUnknown())
         return new character();
      
      return substring(src.value, pos.intValue(), len, null, src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    *
    * @return   The resulting substring or <code>unknown value</code> if the
    *           <code>src</code> is <code>unknown</code>.
    */
   public static character substring(character src, int pos, int len)
   {
      if (src.unknown)
         return new character();
      
      return substring(src.value, pos, len, null, src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * extending for the rest of the string.   Operates on characters rather
    * than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(character src, NumberType pos)
   {
      if (src.unknown || pos.isUnknown())
         return new character();
      
      return substring(src.value, pos.intValue(), -1, null, src.caseSens);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * extending for the rest of the string.  Operates on characters rather
    * than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    *
    * @return   The resulting substring or <code>unknown value</code> if the
    *           <code>src</code> is <code>unknown</code>.
    */
   public static character substring(character src, int pos)
   {
      if (src.unknown)
         return new character();
      
      return substring(src.value, pos, -1, null, src.caseSens);
   }

   /**
    * Returns the substring of the string at the given 1-based index  and
    * extending for the rest of the string. Operates on characters rather than
    * bytes.
    * <p>
    * The returned instance is case-insensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts.
    *
    * @return   The resulting substring.
    */
   public static character substring(String src, int pos)
   {
      return substring(src, pos, -1, null, false);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(String     src,
                                     NumberType pos,
                                     NumberType len)
   {
      if (pos.isUnknown() || len.isUnknown())
         return new character();
      
      return substring(src, pos.intValue(), len.intValue(), null, false);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(String     src,
                                     NumberType pos,
                                     int        len)
   {
      if (pos.isUnknown())
         return new character();
      
      return substring(src, pos.intValue(), len, null, false);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * Case-sensitivity of the returned instance is determined by the
    * <code>src</code> instance case-sensitivity.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string. 
    *
    * @return   The resulting substring or <code>unknown value</code> if any
    *           input is <code>unknown</code>.
    */
   public static character substring(String     src,
                                     int        pos,
                                     NumberType len)
   {
      if (len.isUnknown())
         return new character();
      
      return substring(src, pos, len.intValue(), null, false);
   }
      
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.  Operates on characters rather than bytes.
    * <p>
    * The returned instance is case-insensitive.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    *
    * @return   The resulting substring.
    */
   public static character substring(String src, int pos, int len)
   {
      return substring(src, pos, len, null, false);
   }
   
   /**
    * Returns the substring of the string at the given 1-based index and
    * of the given length.
    *
    * @param    src
    *           The string to operate upon.
    * @param    pos
    *           The 1-based index at which the substring starts. 
    * @param    len
    *           The length of the substring to return. Use -1 for the rest of
    *           the string.
    * @param    type
    *           Specifies whether this substring operates on bytes or
    *           characters.  At this time only "raw" and "character" are
    *           supported.
    * @param    caseSens
    *           Determines the case-sensitivity of the returned instance.
    *
    * @return   The resulting substring.
    */
   public static character substring(String  src,
                                     int     pos, 
                                     int     len,
                                     String  type,
                                     boolean caseSens)
   {
      String result = null;
      
      // convert the Progress 1-based index into Java's 0-based index
      int start = pos - 1;
      
      if (pos < 1)
      {
         genNegativeIndexError();
         return new character("");
      }
      
      if (type != null && "raw".equalsIgnoreCase(type))
      {
         // raw mode (operates on bytes)
         byte[] buffer = src.getBytes(); 
         
         if (start >= buffer.length)
         {
            // requesting data past the end of the string yields the empty
            // string
            result = "";
         }
         else
         {
            int end = start + len;
            
            if (len == -1 || end >= buffer.length)
            {
               len = buffer.length - start;
            }
            
            result = new String(buffer, start, len);         
         }
      }
      else
      {      
         // character mode
         
         if (start >= src.length())
         {
            // requesting data past the end of the string yields the empty
            // string
            result = "";
         }
         else
         {
            int end = start + len;
            
            if (len == -1 || end >= src.length())
            {
               // if the requested data is more than the rest of the string
               // or if the rest of the string was explicitly requested via
               // -1
               result = src.substring(start);
            }
            else
            {
               // only return a portion
               result = src.substring(start, end);
            }            
         }         
      }
      
      return new character(result, caseSens);
   }
      
   /**
    * Returns a character instance that contains the lowercased version of
    * the specified string. This may generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for case 
    * conversion. If this returned value is different than would be returned
    * under Progress AND if that Progress-specific value is used in
    * comparison with this returned value, then a difference in logic will be
    * seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    * <p>
    * If the operand is case-sensitive, the result will be case-sensitive.
    *
    * @param    c
    *           The character instance to convert (cannot be
    *           <code>null</code>).
    *
    * @return   The lowercased character instance (which should match up
    *           with the Progress equivalents in the ASCII range) or
    *           <code>unknown value</code> if the input string is
    *           <code>unknown value</code>.
    */
   public static character toLowerCase(character c)
   {
      if (c.unknown)
         return new character();
      
      character res = toLowerCase(c.value);
      
      // honor case-sensitivity
      res.caseSens = c.caseSens;
      
      return res;
   }
      
   /**
    * Returns a character instance that contains the lowercased version of
    * the specified string. This may generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for case 
    * conversion. If this returned value is different than would be returned
    * under Progress AND if that Progress-specific value is used in
    * comparison with this returned value, then a difference in logic will be
    * seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    * <p>
    * The result will be case-insensitive.
    *
    * @param    str
    *           The string to convert (cannot be <code>null</code>).
    *
    * @return   The lowercased character instance which should match up
    *           with the Progress equivalents in the ASCII range.
    */
   public static character toLowerCase(String str)
   {
      return new character(str.toLowerCase());
   }
      
   /**
    * Returns a character instance that contains the uppercased version of
    * the specified string. This may generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for case 
    * conversion. If this returned value is different than would be returned
    * under Progress AND if that Progress-specific value is used in
    * comparison with this returned value, then a difference in logic will be
    * seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    * <p>
    * If the operand is case-sensitive, the result will be case-sensitive.
    *
    * @param    c
    *           The character instance to convert (cannot be
    *           <code>null</code>).
    *
    * @return   The uppercased character instance (which should match up
    *           with the Progress equivalents in the ASCII range) or
    *           <code>unknown value</code> if the input string is
    *           <code>unknown value</code>.
    */
   public static character toUpperCase(character c)
   {
      if (c.unknown)
         return new character();
      
      character res = toUpperCase(c.value);
      
      // honor case-sensitivity
      res.caseSens = c.caseSens;
      
      return res;
   }
      
   /**
    * Returns a character instance that contains the uppercased version of
    * the specified string. This may generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for case 
    * conversion. If this returned value is different than would be returned
    * under Progress AND if that Progress-specific value is used in
    * comparison with this returned value, then a difference in logic will be
    * seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    * <p>
    *The result will be case-insensitive.
    *
    * @param    str
    *           The string to convert (cannot be <code>null</code>).
    *
    * @return   The uppercased character instance which should match up
    *           with the Progress equivalents in the ASCII range.
    */
   public static character toUpperCase(String str)
   {
      return new character(str.toUpperCase());
   }

   /**
    * Returns a string instance that contains the uppercased version of
    * the specified string. This may generate incorrect results in a
    * DBCS environment since it will use UNICODE as the basis for case
    * conversion. If this returned value is different than would be returned
    * under Progress AND if that Progress-specific value is used in
    * comparison with this returned value, then a difference in logic will be
    * seen.
    * <p>
    * This method does not handle the case of user-specified source and/or
    * target codepages as is possible in Progress.
    * <p>
    *The result will be case-insensitive.
    *
    * @param    str
    *           The string to convert (cannot be <code>null</code>).
    * @param    runtimeString
    *           <code>true</code> if the target string is a runtime string,
    *           <code>false</code> if it is a hardcoded string into a source
    *           file.
    *
    * @return   The uppercased character instance which should match up
    *           with the Progress equivalents in the ASCII range.
    */
   public static String toUpperCase(String str, boolean runtimeString)
   {
      str = str.toUpperCase();

      if (!runtimeString)
      {
         StringBuffer sb = new StringBuffer("");
         int end = str.length();

         for (int i = 0; i < end; i++)
         {
            char current = str.charAt(i);
            char next;

            switch (current)
            {
               case '\\':
                  i++;
                  next = str.charAt(i);
                  sb.append('\\');

                  // change back mistakenly uppercased special characters
                  // \t \r \n \b \f into a hardcoded string
                  switch (next)
                  {
                     case 'T':
                        sb.append('t');
                        break;
                     case 'R':
                        sb.append('r');
                        break;
                     case 'N':
                        sb.append('n');
                        break;
                     case 'B':
                        sb.append('b');
                        break;
                     case 'F':
                        sb.append('f');
                        break;
                     default:
                        sb.append(next);
                  }
                  break;
               default:
                  sb.append(current);
            }
         }

         str = sb.toString();
      }

      return str;
   }

   /**
    * Trims the list of contiguous trim characters from the left of an input
    * string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of only
    * the input parameter, the trim character list case-sensitivity is
    * ignored.  Likewise, if the input parameter is case-sensitive, then
    * the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(character input, character list)
   {
      if (input.unknown || list.unknown)
      {
         return new character();
      }
      
      return leftTrim(input.value, list.value, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left of an input
    * string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * the input parameter. If the input parameter is case-sensitive, then
    * the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(character input, String list)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return leftTrim(input.value, list, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left of an input
    * string.
    * <p>
    * Trim operations are performed case-insensitively.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(String input, character list)
   {
      if (list.unknown)
      {
         return new character();
      }
      
      return leftTrim(input, list.value, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * left of an input string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * input parameter.  Likewise, if the input parameter is case-sensitive,
    * then the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(character input)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return leftTrim(input.value, null, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left of an input
    * string.  If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    * @param    caseSens
    *           Specifies the case-sensitivity of the trim operation AND of
    *           the returned instance.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(String  input,
                                    String  list,
                                    boolean caseSens)
   {
      return new character(trim(input, list, true, caseSens), caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left of an input
    * string.  If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(String input, String list)
   {
      return leftTrim(input, list, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * left of an input string.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character leftTrim(String input)
   {
      return leftTrim(input, (String) null);
   }
   
   /**
    * Trims the list of contiguous trim characters from the right of an input
    * string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of only
    * the input parameter, the trim character list case-sensitivity is
    * ignored.  Likewise, if the input parameter is case-sensitive, then
    * the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(character input, character list)
   {
      if (input.unknown || list.unknown)
      {
         return new character();
      }
      
      return rightTrim(input.value, list.value, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the right of an input
    * string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * the input parameter. If the input parameter is case-sensitive, then
    * the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(character input, String list)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return rightTrim(input.value, list, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the right of an input
    * string.
    * <p>
    * Trim operations are performed based case-insensitively.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(String input, character list)
   {
      if (list.unknown)
      {
         return new character();
      }
      
      return rightTrim(input, list.value, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * right of an input string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * input parameter.  Likewise, if the input parameter is case-sensitive,
    * then the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(character input)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return rightTrim(input.value, null, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the right of an input
    * string.  If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    * @param    caseSens
    *           Specifies the case-sensitivity of the trim operation AND of
    *           the returned instance.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(String  input,
                                     String  list,
                                     boolean caseSens)
   {
      return new character(trim(input, list, false, caseSens), caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the right of an input
    * string.  If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.

    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(String input, String list)
   {
      return rightTrim(input, list, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * right of an input string.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.    
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character rightTrim(String input)
   {
      return rightTrim(input, (String) null);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left and right
    * of an input string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of only
    * the input parameter, the trim character list case-sensitivity is
    * ignored.  Likewise, if the input parameter is case-sensitive, then
    * the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character trim(character input, character list)
   {
      if (input.unknown || list.unknown)
      {
         return new character();
      }
      
      return trim(input.value, list.value, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left and right
    * of an input string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * input parameter. Likewise, if the input parameter is case-sensitive,
    * then the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character trim(character input, String list)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return trim(input.value, list, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left and right
    * of an input string.
    * <p>
    * Trim operations are performed case-insensitively and the returned
    * result will be case-insensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim.
    *
    * @return   The trimmed result.
    */
   public static character trim(String input, character list)
   {
      if (list.unknown)
      {
         return new character();
      }
      
      return trim(input, list.value, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * left and right of an input string.
    * <p>
    * Trim operations are performed based on the case-sensitivity of the
    * input parameter.  Likewise, if the input parameter is case-sensitive,
    * then the returned instance will be case-sensitive.
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character trim(character input)
   {
      if (input.unknown)
      {
         return new character();
      }
      
      return trim(input.value, null, input.caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left and right
    * of an input string. If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    * @param    caseSens
    *           Specifies the case-sensitivity of the trim operation AND of
    *           the returned instance.
    *
    * @return   The trimmed result.
    */
   public static character trim(String input, String list, boolean caseSens)
   {
      // trim left
      String result = trim(input, list, true, caseSens);
      
      // trim right
      return new character(trim(result, list, false, caseSens), caseSens);
   }
   
   /**
    * Trims the list of contiguous trim characters from the left and right
    * of an input string. If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    *
    * @return   The trimmed result.
    */
   public static character trim(String input, String list)
   {
      return trim(input, list, false);
   }
   
   /**
    * Trims any contiguous space, tab, line feed and carriage return from the
    * left and right of an input string.  Trim operations are performed
    * case-insensitively.
    * <p>
    * Trim operations and the returned instance will be case-insensitive.    
    *
    * @param    input
    *           The string to trim.
    *
    * @return   The trimmed result.
    */
   public static character trim(String input)
   {
      return trim(input, (String) null, false);
   }
   
   /**
    * Convert a <code>double</code> into a character object using
    * the 'message' format for the <code>decimal</code> data type.
    *
    * @param    num
    *           The data to convert.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(double num)
   {
      return valueOf(new decimal(num));
   }
   
   /**
    * Convert an <code>int</code> into a character object using
    * the 'message' format for the <code>integer</code> data type.
    *
    * @param    num
    *           The data to convert.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(int num)
   {
      return valueOf(new integer(num));
   }
   
   /**
    * Convert a <code>String</code> into a character object using
    * the 'message' format for the <code>character</code> data type.
    *
    * @param    s
    *           The data to convert.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(String s)
   {
      return valueOf(new character(s));
   }
   
   /**
    * Convert a <code>boolean</code> into a character object using
    * the 'message' format for the <code>logical</code> data type.
    *
    * @param    b
    *           The data to convert.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(boolean b)
   {
      return valueOf(new logical(b));
   }
   
   /**
    * Convert any Progress compatible wrapper into a character object using
    * the 'message' format for the data type being converted.
    *
    * @param    bdt
    *           The data to convert.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(BaseDataType bdt)
   {
      if (bdt.isUnknown())
         return new character();

      return new character(bdt.toStringMessage());
   }   
   
   /**
    * Convert a <code>double</code> into a character object using
    * the given format for the <code>decimal</code> data type.
    *
    * @param    num
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(double num, String fmt)
   {
      return valueOf(new decimal(num), fmt);
   }
   
   /**
    * Convert an <code>int</code> into a character object using
    * the given format for the <code>integer</code> data type.
    *
    * @param    num
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(int num, String fmt)
   {
      return valueOf(new integer(num), fmt);
   }
   
   /**
    * Convert a <code>String</code> into a character object using
    * the given format for the <code>character</code> data type.
    *
    * @param    s
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(String s, String fmt)
   {
      return valueOf(new character(s), fmt);
   }
   
   /**
    * Convert a <code>boolean</code> into a character object using
    * the given format for the <code>logical</code> data type.
    *
    * @param    b
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(boolean b, String fmt)
   {
      return valueOf(new logical(b), fmt);
   }
   
   /**
    * Convert any Progress compatible wrapper into a character object using
    * the given format for the data type being converted.
    *
    * @param    bdt
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(BaseDataType bdt, String fmt)
   {
      if (bdt.isUnknown())
         return new character();

      return new character(bdt.toString(fmt));
   }
   
   /**
    * Convert a <code>double</code> into a character object using
    * the given format for the <code>decimal</code> data type.
    *
    * @param    num
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(double num, character fmt)
   {
      return valueOf(new decimal(num), fmt);
   }
   
   /**
    * Convert an <code>int</code> into a character object using
    * the given format for the <code>integer</code> data type.
    *
    * @param    num
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(int num, character fmt)
   {
      return valueOf(new integer(num), fmt);
   }
   
   /**
    * Convert a <code>String</code> into a character object using
    * the given format for the <code>character</code> data type.
    *
    * @param    s
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(String s, character fmt)
   {
      return valueOf(new character(s), fmt);
   }
   
   /**
    * Convert a <code>boolean</code> into a character object using
    * the given format for the <code>logical</code> data type.
    *
    * @param    b
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(boolean b, character fmt)
   {
      return valueOf(new logical(b), fmt);
   }
   
   /**
    * Convert any Progress compatible wrapper into a character object using
    * the given format for the data type being converted.
    *
    * @param    bdt
    *           The data to convert.
    * @param    fmt
    *           The format string to use.
    *
    * @return   The resulting string representation.
    */
   public static character valueOf(BaseDataType bdt, character fmt)
   {
      return valueOf(bdt, fmt.getValue());
   }
   
   /**
    * Tests a bit in the given position in the input array to determine if
    * that bit is a 0 or 1.  The input string is treated as a multibyte 
    * bitfield (a byte array). The first character of the string corresponds
    * to bit positions 0-7 (if it is non-DBCS and presumably in a DBCS
    * character the positions would be 0-15, although this is not known).  In
    * a non-DBCS charset, the second char would correspond to bits 8-15.
    * <p>
    * This feature supports an undocumented Progress built-in function called
    * _cbit().
    *
    * @param    data
    *           A string with 0 or more characters.
    * @param    index
    *           The bit position to test.
    *
    * @return   The return is always <code>false</code> if the empty string
    *           is input or if a negative index is specified.  For any string
    *           of > 0 bytes length, the return is <code>true</code> if the
    *           bit in the specified position is 1 and <code>false</code> if
    *           it is 0. 
    */
   public static logical testBitAt(character data, integer index)
   {
      if (data.isUnknown() || index.isUnknown())
      {
         return new logical();
      }
      
      return testBitAt(data.getValue(), index.intValue());
   }
    
   /**
    * Tests a bit in the given position in the input array to determine if
    * that bit is a 0 or 1.  The input string is treated as a multibyte 
    * bitfield (a byte array). The first character of the string corresponds
    * to bit positions 0-7 (if it is non-DBCS and presumably in a DBCS
    * character the positions would be 0-15, although this is not known).  In
    * a non-DBCS charset, the second char would correspond to bits 8-15.
    * <p>
    * This feature supports an undocumented Progress built-in function called
    * _cbit().
    *
    * @param    data
    *           A string with 0 or more characters.
    * @param    index
    *           The bit position to test.
    *
    * @return   The return is always <code>false</code> if the empty string
    *           is input or if a negative index is specified.  For any string
    *           of > 0 bytes length, the return is <code>true</code> if the
    *           bit in the specified position is 1 and <code>false</code> if
    *           it is 0. 
    */
   public static logical testBitAt(character data, int index)
   {
      if (data.isUnknown())
      {
         return new logical();
      }
      
      return testBitAt(data.getValue(), index);
   }
    
   /**
    * Tests a bit in the given position in the input array to determine if
    * that bit is a 0 or 1.  The input string is treated as a multibyte 
    * bitfield (a byte array). The first character of the string corresponds
    * to bit positions 0-7 (if it is non-DBCS and presumably in a DBCS
    * character the positions would be 0-15, although this is not known).  In
    * a non-DBCS charset, the second char would correspond to bits 8-15.
    * <p>
    * This feature supports an undocumented Progress built-in function called
    * _cbit().
    *
    * @param    data
    *           A string with 0 or more characters.
    * @param    index
    *           The bit position to test.
    *
    * @return   The return is always <code>false</code> if the empty string
    *           is input or if a negative index is specified.  For any string
    *           of > 0 bytes length, the return is <code>true</code> if the
    *           bit in the specified position is 1 and <code>false</code> if
    *           it is 0. 
    */
   public static logical testBitAt(String data, integer index)
   {
      if (index.isUnknown())
      {
         return new logical();
      }
      
      return testBitAt(data, index.intValue());
   }
    
   /**
    * Tests a bit in the given position in the input array to determine if
    * that bit is a 0 or 1.  The input string is treated as a multibyte 
    * bitfield (a byte array). The first character of the string corresponds
    * to bit positions 0-7 (if it is non-DBCS and presumably in a DBCS
    * character the positions would be 0-15, although this is not known).  In
    * a non-DBCS charset, the second char would correspond to bits 8-15.
    * <p>
    * This feature supports an undocumented Progress built-in function called
    * _cbit().
    *
    * @param    data
    *           A string with 0 or more characters.
    * @param    index
    *           The bit position to test.
    *
    * @return   The return is always <code>false</code> if the empty string
    *           is input or if a negative index is specified.  For any string
    *           of > 0 bytes length, the return is <code>true</code> if the
    *           bit in the specified position is 1 and <code>false</code> if
    *           it is 0. 
    */
   public static logical testBitAt(String data, int index)
   {
      // invalid/empty data or invalid index
      if (data == null || data.length() == 0 || index < 0)
      {
         return new logical(false);
      }
      
      byte[] bits    = data.getBytes();
      int    byteOff = index / 8;
      
      // index is too large 
      if (byteOff >= bits.length)
      {
         return new logical(false);
      }
      
      byte bitOff = (byte) (0xFF & (0x01 << (index % 8)));
      byte result = (byte) (0xFF & (bits[byteOff] & bitOff));
      
      return new logical(result != 0x00); 
   }
   
   /**
    * Convert the Progress 4GL style character type format string into a 
    * form that is easier to process.  All non-escaped 'A(n)' forms are
    * expanded into n instances of A.  All recognized (non-fill) format
    * characters are lowercased in the output to make comparisons easier.
    *
    * @param    fmt
    *           The Progress 4GL format string for a character data type.
    *
    * @return   The simplified/expanded format string.
    */
   public static String preprocessFormatString(String fmt)
   {
      StringBuffer sb = new StringBuffer();
      
      int  len = fmt.length();
      char c   = (char) 0;
      
      char    last;
      boolean nonFill = false;
      
      for (int i = 0; i < len; i++) 
      {
         last = c;
         c    = fmt.charAt(i);
         
         switch (c)
         {
            case 'x':
            case 'X':
            case 'n':
            case 'N':
            case 'a':
            case 'A':
            case '!':
            case '9':
               if (last != '~')
               {
                  // these are recognized valid 
                  nonFill = true;
                  sb.append(Character.toLowerCase(c));
               }
               else
               {
                  // any fill chars just go into the output unchanged
                  nonFill = false;
                  sb.append(c);
               }
               break;
            case '(':
               if (last != '~' && nonFill)
               {  
                  // find the end of the parens
                  int j = fmt.indexOf(')', i + 1);  
                  
                  if (j < 0)
                  {
                     String spec = "Character format %s is incomplete";
                     String err  = String.format(spec, fmt);
                     
                     ErrorManager.recordOrThrowError(164, err);
                     
                     // this is the safest thing to return (any results
                     // should be undone anyway)
                     return "";                     
                  }
                  
                  // how many repeats do we have?
                  int num = Integer.parseInt(fmt.substring(i+1, j));
                  
                  // get our last char
                  char ch = Character.toLowerCase(last);
                  
                  // create and add the resulting string which takes advantage
                  // of the fact that the last char is already in the buffer
                  // and so we only need to repeat (num - 1) times
                  sb.append(StringHelper.repeatChar(ch, num - 1));
                  
                  // move our index up to the end of the section, it will
                  // be incremented past/tested against the string length
                  // by the for loop itself
                  i = j;
               }
               else
               {
                  if (last == '~')
                  {
                     // replace the tilde with this char
                     sb.setCharAt(sb.length() - 1, c);
                  }
                  else
                  {
                     // we must be following a fill char so this is a fill
                     // char too
                     sb.append(c);
                  }
               }
               nonFill = false;
               break;
            default:
               // any fill chars just go into the output unchanged
               nonFill = false;
               sb.append(c);
         }
      }
      
      return sb.toString();
   }

   /**
    * Performs postprocessing of the string literals into HQL expressions.
    * The following conversions will be performed:
    *  '    to    ''
    *  \\   to    \\\\ (runtime); \\\\\\\\ (hardcoded)  
    *
    * @param  text
    *         The string literal to process (without enclosing quotes).
    * @param  runtimeString
    *         <code>true</code> if the target string is a runtime string,
    *         <code>false</code> if it is a hardcoded string into a source
    *         file.
    *
    * @return processed string literal.
    */
   public static String postprocessStringLiteral(String text,
                                                 boolean runtimeString)
   {
      StringBuffer sb = new StringBuffer("");
      int end = text.length();

      for (int i = 0; i < end; i++)
      {
         char current = text.charAt(i);

         switch (current)
         {
            case '\'':
               sb.append("\'\'");
               break;

            case '\\':
               if (runtimeString)
               {
                  sb.append("\\\\");
               }
               else
               {
                  i++;
                  char next = text.charAt(i);
                  if (next == '\\')
                  {
                     sb.append("\\\\\\\\");
                  }
                  else
                  {
                     sb.append(current);
                     sb.append(next);
                  }
                  break;
               }

            default:
               sb.append(current);
         }
      }

      return sb.toString();
   }
   
   /**
    * Calculate the length of a formatted value (the text form of the value)
    * using this format string.
    *
    * @param    fmt
    *           The format string.
    *
    * @return   The length in characters of the any resulting formatted
    *           value.
    */
   public int formatLength(String fmt)
   {
      if (fmt == null)
      {
         fmt = defaultFormatString();
      }
      
      return super.formatLength(preprocessFormatString(fmt));
   }
   
   /**
    * Hash code implementation which is consistent with {@link
    * BaseDataType#equals}.
    *
    * @return  Hash code value for this object instance.
    */
   public int hashCode()
   {
      int result = 17;
      if (isUnknown())
      {
         return result;
      }
      
      String s = (caseSens ? value : value.toUpperCase());
      result = 37 * result + s.hashCode();
      
      return result;
   }
   
   /**
    * Hash code implementation which is consistent with {@link
    * #equalsCaseSensitively}.
    *
    * @return  Hash code value for this object instance.
    */
   public int caseSensitiveHashCode()
   {
      int result = 17;
      if (isUnknown())
      {
         return result;
      }
      
      result = 37 * result + value.hashCode();
      
      return result;
   }
   
   /**
    * Creates a new instance of the same type that represents the 
    * <code>unknown value</code>.
    *
    * @return   An instance that represents the <code>unknown value</code>.
    */
   public BaseDataType instantiateUnknown()   
   {
      return new character();
   }
   
   /**
    * Creates a new instance of the same type that represents the 
    * default initialized value.
    *
    * @return   An instance that represents the default value.
    */
   public BaseDataType instantiateDefault()   
   {
      return new character("");
   }
   
   /**
    * Does the same as standard <code>clone()</code> method but returns an
    * instance of <code>BaseDataType</code> and doesn't throw the
    * <code>CloneNotSupportedException</code>.
    *
    * @return   A clone of this instance.  
    */
   public BaseDataType duplicate()
   {
      return new character(this);
   }
   
   /**
    * Determines if this instance represents the <code>unknown value</code>.
    *
    * @return   <code>true</code> if this instance is set to the
    *           <code>unknown value</code>.
    */
   public boolean isUnknown()
   {
      return unknown;
   }
   
   /**
    * Sets the state of this instance's <code>unknown value</code> flag to
    * <code>true</code>.
    * <p>
    * <b>Warning: a separate call is needed to ensure that the data of this
    * instance is set to the correct value.</b>
    */
   public void setUnknown()
   {
      unknown = true;
   }
   
   /**
    * Determines if this instance compares case sensitively.
    *
    * @return   <code>true</code> if this instance is set to case sensitive.
    */
   public boolean isCaseSensitive()
   {
      return caseSens;
   }
   
   /**
    * Sets the state of this instance's compares case sensitive flag.
    * <p>
    * This method is used to set the current value in a manner that does not
    * schedule an automated reset to the prior value later. Thus, this method
    * is suitable for the initial configuration of a new instance.  If a
    * temporary case-sensitivity change is needed, use 
    * {@link #setTemporaryCaseSensitive}.
    *
    * @param    caseSens
    *           <code>true</code> if this instance must be set to case
    *           sensitive.
    */
   public void setCaseSensitive(boolean caseSens)
   {
      this.caseSens = caseSens;
   }
   
   /**
    * Sets the state of this instance's compares case sensitive flag for the
    * duration of the caller's current scope.  At the end of the caller's
    * scope a notification will trigger a reset of the case sensitive flag
    * to the current value.
    * <p>
    * This method should only be used when importing references via the
    * {@link SharedVariableManager} as a local instance or a new shared
    * instance would not need to be reset, since they would have no prior
    * value to which to return.  To set the initial case-sensitivity, please
    * se {@link #setCaseSensitive}.
    * <p>
    * The {@link TransactionManager#registerFinalizable} facility is used to
    * obtain the callback necessary to reset the sensitivity at end of the
    * caller's scope.
    *
    * @param    caseSens
    *           <code>true</code> if this instance must be set to case
    *           sensitive.
    */
   public void setTemporaryCaseSensitive(boolean caseSens)
   {
      // do nothing if the request precision is the same as the current
      // precision
      if (this.caseSens != caseSens)
      {
         // remember the old value
         boolean old = this.caseSens;
         
         // reset
         setCaseSensitive(caseSens);
         
         // schedule the reset
         SensitivityResetter sr = new SensitivityResetter(old);
         TransactionManager.registerFinalizable(sr, false);
      }
   }
   
   /**
    * Accesses the state of this instance's <code>String</code> value
    * independently of the state of whether this instance represents the
    * <code>unknown value</code>.  If this instance represents the
    * <code>unknown value</code>, the returned <code>String</code> is
    * <b>not valid</b>! 
    *
    * @return   The state of this instance's <code>String</code> value.
    */
   public String getValue()
   {
      return value;
   }
   
   /**
    * Sets the state of this instance's <code>String</code> value
    * and sets the <code>unknown value</code> state for this instance to
    * <code>false</code>.
    *
    * @param    value
    *           The state to set this instance's <code>String</code> value.
    */
   public void setValue(String value)
   {
      this.value = value;
      unknown    = false;
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance.
    *
    * @param    value
    *           The instance from which to copy state.
    * @param    force
    *           Flag to indicate if a pending error should be ignored (see 
    *           {@link ErrorManager#isPendingError} and the assignment
    *           forced regardless.  Note that normal assignment silently
    *           exits with no change if there is a pending error. If this
    *           flag is <code>true</code>, that check is bypassed and the
    *           assignment will always be done.
    */
   public void assign(BaseDataType value, boolean force)
   {
      assign((character) value, force);
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance.
    *
    * @param    value
    *           The instance from which to copy state.
    * @param    force
    *           Flag to indicate if a pending error should be ignored (see 
    *           {@link ErrorManager#isPendingError} and the assignment
    *           forced regardless.  Note that normal assignment silently
    *           exits with no change if there is a pending error. If this
    *           flag is <code>true</code>, that check is bypassed and the
    *           assignment will always be done.
    */
   public void assign(character value, boolean force)
   {
      if (force || !ErrorManager.isPendingError())
      {
         this.value = value.value;
         unknown    = value.unknown;
      }
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance. The configuration (case-sensitivity) of
    * this instance is not set.  Use {@link #deepAssign} if both state and
    * configuration must be set.
    * <p>
    * This is a "no operation" in the case where silent error mode is active
    * AND an error has occurred in the current processing since the opening
    * call to {@link ErrorManager#silentErrorEnable}. This protection is
    * needed to ensure that the state of this instance does not change in
    * this case.  This will not affect undo because that only occurs when
    * silent error mode is disabled.
    *
    * @param    value
    *           The instance from which to copy state. If <code>null</code>
    *           is passed, the instance will be set to the unknown value.
    */
   public void assign(String value)
   {
      assign(new character(value), false);
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance. The configuration (case-sensitivity) of
    * this instance is not set.  Use {@link #deepAssign} if both state and
    * configuration must be set.
    * <p>
    * This is a "no operation" in the case where silent error mode is active
    * AND an error has occurred in the current processing since the opening
    * call to {@link ErrorManager#silentErrorEnable}. This protection is
    * needed to ensure that the state of this instance does not change in
    * this case.  This will not affect undo because that only occurs when
    * silent error mode is disabled.
    *
    * @param    value
    *           The instance from which to copy state.
    */
   public void assign(character value)
   {
      assign(value, false);
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the
    * state of the passed instance.
    *
    * @param    value
    *           The instance from which to copy state.
    */
   public void assign(Undoable value)
   {
      assign((character) value, false);
   }
   
   /**
    * Sets the state (data and unknown value) and configuration 
    * (case-sensitivity) of this instance based on the state and configuration
    * of the passed instance.
    *
    * @param    value
    *           The instance from which to copy state and configuration.
    */
   public void deepAssign(character value)
   {
      assign(value, true);
      caseSens = value.caseSens;
   }
   
   /**
    * Compares this instance with the specified instance and returns a -1
    * if this instance is less than the specified, 0 if the two instances
    * are equal and 1 if this instance is greater than the specified
    * instance.  This is the implementation of the <code>Comparable</code>
    * interface.
    * <p>
    * Core worker that handles comparisons between 2 string operands after
    * removing trailing whitespace and honoring the case sensitivity 
    * operands.  
    * <p>
    * Case-insensitive comparisons are obtained by manually uppercasing both
    * operands which is how Progress does it. Lowercase comparisons cannot be 
    * used becuase the results for {@link CompareOps#isGreaterThan},
    * {@link CompareOps#isLessThan}, {@link CompareOps#isGreaterThanOrEqual}
    * and {@link CompareOps#isLessThanOrEqual} would be different in Java
    * than in Progress when the following characters are included in the
    * string:
    * <pre>
    * [
    * \
    * ^
    * _
    * '
    * </pre>
    * <p>
    * The algorithm will fail to give meaningful results in the case where
    * one tries to sort against other objects that do not represent compatible
    * values.
    *
    * @param    obj
    *           The instance to compare against.
    *
    * @return   -1, 0 or 1 depending on if this instance is less than,
    *           equal to or greater (respectively) than the specified
    *           instance <code>obj</code>.
    */
   public int compareTo(Object obj)   
   {
      character c = null;
      
      if (obj instanceof character)
      {
         c = (character) obj;
      }
      else if (obj instanceof String)
      {
         c = new character((String) obj);
      }
      else
      {
         return -1;
      }
      
      // prepare the data
      String s1 = StringHelper.safeTrimTrailing(value);      
      String s2 = StringHelper.safeTrimTrailing(c.value);
      
      // if either instance requires case sensitivity, force a case
      // sensitive comparison
      if (caseSens || c.caseSens)
      {
         
         return s1.compareTo(s2);
      }
               
      // DO NOT use String.compareToIgnoreCase() since this lowercases
      // and yields different results for >, <, >= and <= forms when
      // [ \ ^ _ ' characters are included in the operands
      return s1.toUpperCase().compareTo(s2.toUpperCase());
   }
   
   /**
    * Test equality between the internal values of this object and the given
    * parameter on a case sensitive basis.  Before the internal values are
    * checked, the unknown states of both operands are compared.  If both are
    * unknown, <code>true</code> is returned.  If only one is unknown,
    * <code>false</code> is returned.  Otherwise, the case sensitive test is
    * performed.
    *
    * @param   c
    *          Another character instance to test for case sensitive equality
    *          with this object.
    *
    * @return  <code>true</code> if this object and the given object test as
    *          equal, else <code>false</code>.
    */
   public boolean equalsCaseSensitively(character c)
   {
      boolean u1 = isUnknown();
      boolean u2 = c.isUnknown(); 
      
      if (u1 && u2) 
         return true;
      
      if (u1 || u2)
         return false;
      
      // prepare the data
      String s1 = StringHelper.safeTrimTrailing(value);      
      String s2 = StringHelper.safeTrimTrailing(c.value);
      
      return s1.equals(s2);
   }
   
   /**
    * Returns the larger (lexicographically) of the current or passed-in
    * instance.
    *
    * @param    c
    *           The instance to compare against.
    *
    * @return   The largest instance.
    */
   public character maximum(character c)   
   {
      return (character) super.maximum(c);
   }
      
   /**
    * Returns the smaller (lexicographically) of the current or passed-in 
    * instance.
    *
    * @param    c
    *           The instance to compare against.
    *
    * @return   The smallest instance.
    */
   public character minimum(character c)   
   {
      return (character) super.minimum(c);
   }   
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i,
                                 character  delimit,
                                 character  replace)
   throws ErrorConditionException
   {
      if (i.isUnknown() || replace.unknown || delimit.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(),
                          safeDelimiter(delimit.value),
                          replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(double    i,
                                 character delimit,
                                 character replace)
   throws ErrorConditionException
   {
      if (replace.unknown || delimit.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry((int) i,
                          safeDelimiter(delimit.value),
                          replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(double    i,
                                 character delimit,
                                 String    replace)
   throws ErrorConditionException
   {
      if (delimit.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry((int) i, safeDelimiter(delimit.value), replace);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i,
                                 character  delimit,
                                 String     replace)
   throws ErrorConditionException
   {
      if (i.isUnknown() || delimit.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(),
                          safeDelimiter(delimit.value),
                          replace);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i,
                                 String     delimit,
                                 character  replace)
   throws ErrorConditionException
   {
      if (i.isUnknown() || replace.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(),
                          safeDelimiter(delimit),
                          replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(double i, String delimit, character replace)
   throws ErrorConditionException
   {
      if (replace.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry((int) i, safeDelimiter(delimit), replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i,
                                 String     delimit,
                                 String     replace)
   throws ErrorConditionException
   {
      if (i.isUnknown())
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(), safeDelimiter(delimit), replace);
   }
   
   /**
    * Replaces the specified 1-based indexed comma-delimited entry in the
    * current instance which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i, character replace)
   throws ErrorConditionException
   {
      if (i.isUnknown() || replace.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(), ',', replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed comma-delimited entry in the
    * current instance which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(NumberType i, String replace)
   throws ErrorConditionException
   {
      if (i.isUnknown())
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry(i.intValue(), ',', replace);
   }
   
   /**
    * Replaces the specified 1-based indexed comma-delimited entry in the
    * current instance which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.  It may have been set to the 
    *           <code>unknown value</code> if any of the inputs are
    *           <code>unknown value</code>.
    */
   public character replaceEntry(double i, character replace)
   throws ErrorConditionException
   {
      if (replace.unknown)
      {
         setUnknown();
         return this;
      }
      
      return replaceEntry((int) i, ',', replace.value);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.
    */
   public character replaceEntry(double i, String delimit, String replace)
   throws ErrorConditionException
   {
      return replaceEntry((int) i, safeDelimiter(delimit), replace);
   }
   
   /**
    * Replaces the specified 1-based indexed entry in the current instance
    * which must be a delimited list.  
    *
    * @param    ii
    *           The 1-based index into the list.
    * @param    delimit
    *           The delimiter character.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.
    */
   public character replaceEntry(double ii, char delimit, String replace)
   throws ErrorConditionException
   {
      boolean content  = (value.length() > 0); 
      boolean endDelim = content && 
                         (value.charAt(value.length() - 1) == delimit); 
      
      // create the regular expression (with proper escapes)
      String regex = fixupRegexChar(delimit);      
      
      // split up the list (make sure all possible entries are included,
      // even empty trailing entries --> use -1 as the limit to force this) 
      String[] array = value.split(regex, -1);
      
      // convert 1 based index to 0 based
      int i   = (int) ii;
      int idx = i - 1;
      
      if (idx < 0 || idx >= array.length)
      {
         genOutsideRangeError(i, value);
         return this;
      }
      
      StringBuffer sb = new StringBuffer();
      
      // put the list back together
      for (int j = 0; j < array.length; j++)
      {
         // add the delimiter on all subsequent iterations
         if (j > 0)
         {
            sb.append(delimit);
         }
         
         if (j == idx)
         {
            // replace this element
            sb.append(replace);
         }
         else
         {
            // emit this element (even if empty)
            sb.append(array[j]);
         }
      }
      
      // add an ending delimiter if there was one to start
      if (endDelim)
      {
         sb.append(delimit);
      }
      
      // save the result
      value = sb.toString();
      
      return this;
   }
   
   /**
    * Replaces the specified 1-based indexed comma-delimited entry in the
    * current instance which must be a delimited list.  
    *
    * @param    i
    *           The 1-based index into the list.
    * @param    replace
    *           The replacment text.
    *
    * @return   The current instance.
    */
   public character replaceEntry(double i, String replace)
   throws ErrorConditionException
   {
      return replaceEntry((int) i, ',', replace);
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, double pos)
   throws ErrorConditionException
   {
      replaceWorker(new character(text),
                    new integer(pos),
                    new integer(-1),
                    true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, double pos)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), new integer(-1), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, NumberType pos)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, new integer(-1), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, NumberType pos)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, new integer(-1), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, double pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text),
                    new integer(pos),
                    new integer(len),
                    true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, NumberType pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, double pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), new integer(pos), len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, double pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, NumberType pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, double pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, NumberType pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, NumberType pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String text, double pos, double len, String type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(new character(text),
                    new integer(pos),
                    new integer(len),
                    true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String     text,
                            NumberType pos,
                            double     len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(new character(text), pos, new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String     text,
                            double     pos,
                            NumberType len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(new character(text), new integer(pos), len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character text, double pos, double len, String type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(text, new integer(pos), new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(String     text,
                            NumberType pos,
                            NumberType len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(new character(text), pos, len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character  text,
                            double     pos,
                            NumberType len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(text, new integer(pos), len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character  text,
                            NumberType pos,
                            double     len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(text, pos, new integer(len), true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace
    *           and the number of characters from the replacement text to 
    *           include in the replacement.
    * @param    type
    *           Description of how to interpret the position and length values.
    *           At this time only "character" is supported.
    *
    * @return   The current instance is returned.
    */
   public character overlay(character  text,
                            NumberType pos,
                            NumberType len,
                            String     type)
   throws ErrorConditionException
   {
      checkOverlayType(type);
      replaceWorker(text, pos, len, true);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, double pos)
   throws ErrorConditionException
   {
      replaceWorker(new character(text),
                    new integer(pos),
                    new integer(-1),
                    false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, double pos)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), new integer(-1), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, NumberType pos)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, new integer(-1), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, NumberType pos)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, new integer(-1), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, double pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text),
                    new integer(pos),
                    new integer(len),
                    false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, NumberType pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, new integer(len), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, double pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), new integer(pos), len, false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, double pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), new integer(len), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(String text, NumberType pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(new character(text), pos, len, false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, double pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(text, new integer(pos), len, false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, NumberType pos, double len)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, new integer(len), false);
      return this;
   }
   
   /**
    * Overwrites characters in the current value of this instance with the 
    * passed text at the 1-based index into the string.  The entire
    * replacement text will be placed into the resulting value.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    *
    * @return   The current instance is returned.
    */
   public character replace(character text, NumberType pos, NumberType len)
   throws ErrorConditionException
   {
      replaceWorker(text, pos, len, false);
      return this;
   }
   
   /**
    * Returns a clone of the current instance which has all <code>null</code>
    * characters removed from its data (in particular, the data is truncated 
    * starting at any <code>null</code> character).
    *
    * @return   An instance which has no embedded <code>null</code>
    *           characters.
    */
   public character safeDuplicate()
   {
      character ch = (character) duplicate();
      
      if (!unknown)
      {
         ch.value = safeValue();
      }
      
      return ch;
   }
   
   /**
    * Creates a string representation of the instance data with the default
    * Progress 4GL format of 'x(8)'. If the instance represents the
    * <code>unknown value</code>, a '?' will be returned.
    *
    * @return   The formatted string.
    */
   public String toString()   
   {
      return toString(null);
   }
   
   /**
    * Creates a string representation of the instance data in a form that is
    * compatible with the <code>MESSAGE</code> language statement.  If the
    * instance represents the <code>unknown value</code>, a '?' will be
    * returned.
    *
    * @return   The 'message' formatted string.
    */
   public String toStringMessage()   
   {
      // quick-out when unknown
      if (unknown)
         return "?";
      
      return javaTruncateNull(getValue());
   }
   
   /**
    * Creates a string representation of the instance data using the 'export'
    * format.  For the character data type the entire string is returned
    * enclosed in a pair of double quotes.  If the instance represents the
    * <code>unknown value</code>, a '?' will be returned.  Any instances of
    * double quotes inside the string will be doubled up on output.
    *
    * @return   The 'export' formatted string.
    */
   public String toStringExport()   
   {
      // quick-out when unknown (no user override for that case)
      if (unknown)
         return "?";
      
      // open the string
      StringBuffer sb = new StringBuffer("\"");
      
      // truncate at any embedded null
      String txt = javaTruncateNull(value);
      
      // add the contents, doubling up all instance of double quote chars
      sb.append(txt.replaceAll("\"", "\"\""));
      
      // close the string
      sb.append("\"");
      
      return sb.toString();
   }
   
   /**
    * Creates a string representation of the instance data using the user-
    * specified Progress 4GL format string.  If the passed-in format is
    * <code>null</code>, then a default format string of 'x(8)' will be
    * used.  If the instance represents the <code>unknown value</code>,
    * a '?' will be returned.
    *
    * @param    fmt
    *           The Progress 4GL format string for a character data type.
    *
    * @return   The formatted string.
    */
   public String toString(String fmt)   
   {
      // quick-out when unknown (no user override for that case)
      if (unknown)
         return "?";
      
      // implement 'default' behavior 
      if (fmt == null)
      {
         fmt = defaultFormat;
      }
      
      // make the format string easier to process
      String format = preprocessFormatString(fmt);
      
      // handle embedded nulls and trim trailing whitespace off the value
      String clean = StringHelper.safeTrimTrailing(javaTruncateNull(value));
      
      // build the output string and return it
      return processDataString(clean, format);
   }
   
   /**
    * Return the current value as a Java wrapper.
    *
    * @return  Current value or <code>null</code> if unknown.
    */
   public String toJavaType()
   {
      return (isUnknown() ? null : getValue());
   }
   
   /**
    * Return the default display format string for this type.
    *
    * @return   The default format string.
    */
   public String defaultFormatString()   
   {
      return defaultFormat;
   }   
      
   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      unknown  = in.readBoolean();
      caseSens = in.readBoolean();
      
      // the next object can be null so we read an indicator var first to
      // determine if there is following data to be read
      
      if (in.readBoolean())
         value = (String) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeBoolean(unknown);
      out.writeBoolean(caseSens);
      
      // the next object can be null so we write an indicator var first to
      // notify reader if there is following data to be read
      
      boolean there = (value != null); 
      out.writeBoolean(there);
      if (there)
         out.writeObject(value);
   }
   
   /**
    * Check the overlay type description and throw an exception if the type
    * is not yet supported. The check is case-insensitive.
    *
    * @param    type
    *           Must be "character" at this time.
    *
    * @throws   IllegalArgumentException
    *           If the type doesn't match "character".
    */
   private void checkOverlayType(String type)
   {
      if (type != null && !type.equalsIgnoreCase("character"))
      {
         String err = String.format("OVERLAY type %s is not supported!", type);
         throw new IllegalArgumentException(err);
      }
   }
   
   /**
    * Returns the instance's value safely truncated at any <code>null</code>
    * byte.
    *
    * @return   A value with no <code>null</code> bytes.
    */
   private String safeValue()
   {
      return javaTruncateNull(value);
   }
   
   /**
    * Core method to implement overlay and replace support. This method
    * changes the current value of this instance, overwriting characters
    * at the specified index given by <code>pos</code> with the text
    * specified by <code>text</code>.  The number of replaced characters
    * is specified by <code>len</code>, with a negative number indicating
    * that the length of characters in <code>text</code> is used.  If
    * <code>overlay</code> mode is requested, then the <code>len</code> 
    * parameter also limits the number of characters in the <code>text</code>
    * parameter which will be used as the replacement.  If any of the
    * parameters are <code>unknown value</code>, the instance will be set
    * to the <code>unknown value</code>.
    *
    * @param    text
    *           Replacement text.
    * @param    pos
    *           The 1-based index into the string at which the replacement
    *           is to occur.
    * @param    len
    *           The number of characters in the source string to replace.
    * @param    overlay
    *           <code>true</code> if overlay mode is required.
    */
   private void replaceWorker(character  text,
                              NumberType pos,
                              NumberType len,
                              boolean    overlay)
   throws ErrorConditionException
   {
      // quick out
      if (this.isUnknown())
      {
         // nothing to do, you can't replace chars in the unknown value
         return;
      }
      
      // are any of our inputs unknown?
      if (text.isUnknown() || pos.isUnknown() || len.isUnknown())
      {
         // set our result to unknown and return
         setUnknown();
         return;
      }
      
      String rtxt = text.getValue();
      int    rpos = pos.intValue();
      int    rlen = len.intValue();
      
      // argument check
      if (rpos < 1)
      {
         genNegativeIndexError();
         return;
      }
      
      // convert from 1-based to 0-based index
      rpos--;
      
      // -1 is equivalent of the length of the replacement text
      if (rlen == -1)
      {
         rlen = rtxt.length();
      }
      
      // replacement text is exactly as passed-in if not in overlay mode
      if (overlay)
      {
         // overlay mode means that the len parameter also changes the 
         // replacement text
         if (rtxt.length() < rlen)
         {
            // pad to the requested length
            rtxt = StringHelper.leftAlignText(rtxt, rlen);
         }
         else
         {
            // truncate the replacement text
            rtxt = rtxt.substring(0, rlen); 
         } 
      }
      
      StringBuffer sb = new StringBuffer();
      
      // start with the beginning of the string
      if (rpos >= value.length())
      {
         // pad to the requested position
         sb.append(StringHelper.leftAlignText(value, rpos));
      }
      else
      {
         // just substring the first portion
         sb.append(value.substring(0, rpos));
      }
      
      // insert our replacement text
      sb.append(rtxt);
      
      // index of the end of the string
      int elen = rpos + rlen;
      
      // if there is any text at the end of the string, add that
      if (value.length() > elen) 
      {
         sb.append(value.substring(elen));
      }
      
      // save the results into our instance data
      value = sb.toString();
   }
   
   /**
    * Convert the input data into an output string that is formatted as
    * specified by the Progress 4GL style character type format string.
    * <p>
    * Recognized format specifier characters are x, X, n, N, a, A, 9 and
    * !. The presence of any unescaped (with tilde) instance of one of these
    * characters in the format string means that should there be a
    * corresponding character (at the same format specifier index) in the
    * data string, it will be copied to the output string at that same
    * position.  No transformation of the data is done except for the !
    * specifier which causes the data character to be uppercased before
    * output.
    * <p>
    * Any non-recognized format specifier and any escaped character
    * (including escaped format specifiers) are considered user-defined
    * "fill" characters.  These characters are copied from the format
    * string (not the data string) into the output string.  They are copied
    * unchanged.  Any unescaped tildes are dropped (not copied to the
    * output).
    * <p>
    * The input data will be copied to a location in the output string based
    * on the simple index position in the input data AND the relative position
    * of each format specifier.  The presence of fill characters does not
    * change the relative position of format specifier characters. This means
    * that the 1st character in the input string will be copied on output
    * to the location of the 1st format specifier character in the format
    * string, which may not be the first position in the output string (if
    * there are fill characters in the format string before the first format
    * specifier). The 2nd character in the input string will be copied on
    * output to the location of the 2nd format specifier, which may not
    * immediately follow the 1st data character in the output string if there
    * are intervening fill characters between the first and second format
    * specifiers.
    * <p>
    * If there is less data characters (in the input string) than format
    * specifier characters (in the format string), then a ' ' (space) will
    * be output for each trailing format specifier position which has no
    * data to be copied.  This is a kind of padding.
    * <p>
    * If there are more data characters (in the input string) than format
    * specifier characters (in the format string), then the output string
    * will be truncated, not copying those extra data characters (in the
    * input string) to the output string.
    * <p>
    * The output string length will always be equivalent to the size of the
    * format string less any dropped tilde escape characters.
    *
    * @param    data
    *           The input string.
    * @param    fmt
    *           The Progress 4GL format string for a character data type.
    *
    * @return   The output (formatted) string.
    */
   private String processDataString(String data, String fmt)
   {
      StringBuffer sb = new StringBuffer();
      
      int  dlen = data.length();
      int  flen = fmt.length();
      char c    = (char) 0;
      
      char last;
                                                               
      // iterate through all characters in the format string (the result
      // will be the same length as the *real* number of characters in the
      // format string --> unescaped tildes don't count) 
      for (int j = 0, i = 0; i < flen; i++) 
      {
         // remember the last character to track if it is a tilde 
         last = c;
         
         // get next format string character
         c = fmt.charAt(i);
         
         switch (c)
         {
            case '~':
               if (last == '~')
               {
                  // we leave a single tilde in the output and reset our
                  // character to ensure that tilde processing is reset
                  sb.append(c);
                  c = (char) 0;
               }
               break;
            case 'x':
            case 'X':
            case 'n':
            case 'N':
            case 'a':
            case 'A':
            case '9':
            case '!':
               // these are recognized valid format characters
               if (last == '~')
               {
                  // this was escaped so it is a fill char, this just goes
                  // into the output unchanged
                  sb.append(c);
               }
               else
               {
                  if (j < dlen)
                  {
                     // there is still data to output
                     
                     // only the ! causes the data to be transformed before
                     // output, all others cause the unchanged char to be
                     // output
                     char out = data.charAt(j);
                     
                     if (c == '!')
                        out = Character.toUpperCase(out);
                     
                     sb.append(out);
                     j++;
                  }
                  else
                  {
                     // no more data, append a space to pad the end
                     sb.append(' ');
                  }
               }
               break;
            default:
               // must be an unescaped fill char, add to the output unchanged
               sb.append(c);
         }
      }
      
      return sb.toString();
   }
   
   /**
    * Generate an error for a substring or overlay index that is negative.
    */
   private static void genNegativeIndexError()
   {
      String err = "Starting position for SUBSTRING, OVERLAY, etc. must" +
                   " be 1 or greater";
      ErrorManager.recordOrThrowError(82, err);
   }
   
   /**
    * Generate an error for a value out of range.
    *
    * @param    idx
    *           The index that is out of range.
    * @param    list
    *           The list being indexed.
    */
   private static void genOutsideRangeError(int idx, String list)
   {
      String spec = "Entry %d is outside the range of list %s";
      String err  = String.format(spec, idx, list);
      
      ErrorManager.recordOrThrowError(560, err);
   }
   
   /**
    * Converts a single character into a regular expression, including
    * escaping any of the special characters that have other meaning in
    * regular expressions.
    *
    * @param    ch
    *           The exact character on which to match.
    *
    * @return   The resulting regular expression.
    */
   private static String fixupRegexChar(char ch)
   {
      StringBuffer regex = new StringBuffer();
      
      if (ch == '(' || ch == ')' || ch == '.' || ch == '*' || ch == '[' ||
          ch == ']' || ch == '^' || ch == '{' || ch == '}' || ch == '?' ||
          ch == '+' || ch == '?' || ch == '$' || ch == '|' || ch == '\\')
      {
         regex.append('\\').append(ch);
      }
      else
      {
         regex.append(ch);
      }
      
      return regex.toString();
   }
   
   /**
    * Trims the list of contiguous trim characters from the left or right of
    * an input string.  If the list of trim characters is <code>null</code>
    * the method defaults to space, tab, line feed and carriage return.
    *
    * @param    input
    *           The string to trim.
    * @param    list
    *           The list of characters to trim or <code>null</code> to
    *           use the default list.
    * @param    left
    *           Specifies if the trim should occur on the left or right
    *           side of the string.
    * @param    caseSens
    *           If <code>true</code>, a case-sensitive trim is performed.
    *
    * @return   The trimmed result.
    */
   private static String trim(String  input,
                              String  list,
                              boolean left,
                              boolean caseSens)
   {
      // implement our default list
      if (list == null)
      {
         list = " \t\n\r";
      }
      
      int len  = input.length();
      int llen = list.length();
      int i;
      
      for (i = left ? 0 : len - 1; left ? i < len : i >= 0;)
      {
         boolean found = false;
         char    c     = input.charAt(i);
         char    cu    = Character.toUpperCase(c);
         
         for (int j = 0; j < llen; j++)
         {
            char ch  = list.charAt(j);
            char chu = Character.toUpperCase(ch);
            
            if ((caseSens && ch == c) || (!caseSens && chu == cu))  
            {
               found = true;
               break;
            }
         }
         
         if (!found)
         {
            break;
         }
         
         if (left)
         {
            i++;
         }
         else
         {
            i--;
         }
      }
      
      return left ? input.substring(i) : input.substring(0, i + 1);
   }
   
   /**
    * Test for certain failure conditions and if OK return the indexed
    * character, otherwise throw an exception with useful debugging
    * information.
    *
    * @param    input
    *           The string to read from.
    * @param    index
    *           The 0-based index into the string, from which to read the 
    *           character.
    *
    * @return   The character at the specified index.
    *
    * @throws   IllegalArgumentException
    *           If the index is out of bounds.
    */
   private static char readChar(String input, int index)
   throws IllegalArgumentException
   {
      char next;
      
      if (index == input.length())
      {
         throw new IllegalArgumentException("Unexpected end of string.");
      }
      
      try
      {
         next = input.charAt(index); 
      }
      
      catch (IndexOutOfBoundsException iob)
      {
         throw new IllegalArgumentException("Invalid index (" + index +
                                            " in string \"" + input + "\").");
      }
      
      return next; 
   }
   
   /**
    * Given a delimiter string, extract the first character and retrun it.  If
    * the string is empty, return a space character.
    * 
    * @param   delimiter
    *          Delimiter string
    * 
    * @return  Delimiter character as described above.
    */
   private static char safeDelimiter(String delimiter)
   {
      return (delimiter.length() == 0 ? ' ' : delimiter.charAt(0));
   }
   
   /**
    * Helper to reset the associated instance's case sensitivity to a prior
    * value based on a notification that a particular scope has exited.  This
    * scope notification support is provided by 
    * {@link TransactionManager#registerFinalizable} and the {@link
    * Finalizable} interface.
    */
   private class SensitivityResetter
   implements Finalizable
   {
      /** The sensitivity to which the containing instance will be reset. */ 
      boolean oldCaseSens;
      
      /**
       * Construct an instance which can reset the containing instance's
       * case sensitivity when a notification is received.  See 
       * {@link character#setTemporaryCaseSensitive}.
       *
       * @param    oldCaseSens
       *           The case sensitivity to which the containing instance must
       *           be reset.
       */
      SensitivityResetter(boolean oldCaseSens)
      {
         this.oldCaseSens = oldCaseSens;
      }
      
      /**
       * Processes the end of scope notification and resets the associated 
       * instance's case sensitivity.  This notification is provided by the
       * {@link TransactionManager}.
       */
      public void finished()
      {
         character.this.caseSens = oldCaseSens;
      }
      
      /**
       * Processes the iterate notification provided by the {@link
       * TransactionManager}.  This implementation does nothing.
       */
      public void iterate()
      {
      }
      
      /**
       * Processes the retry notification provided by the {@link
       * TransactionManager}.  This implementation does nothing.
       */
      public void retry()
      {
      }      
   }
      
   /**
    * Encapsulates the logic of storing and executing a set of tests using
    * a minimum set of input (2 string operands and a set of boolean results
    * that are to be expected).
    */
   private static class TestCase
   {
      /** Operand 1. */
      character op1 = null;
      
      /** Operand 2. */
      character op2 = null;
      
      /** Expected test results for begins. */
      boolean begins;
      
      /** Expected test results for equals. */
      boolean eq;
      
      /** Expected test results for notEquals. */
      boolean ne;
      
      /** Expected test results for isGreaterThan. */
      boolean gt;
      
      /** Expected test results for isLessThan. */
      boolean lt;
      
      /** Expected test results for isGreaterThanOrEqual. */
      boolean ge;
      
      /** Expected test results for isLessThanOrEqual. */
      boolean le;
      
      /** Expected test results for matches. */
      boolean matches;
      
      /** Expected test results for case-sensitive begins. */
      boolean cBegins;
      
      /** Expected test results for case-sensitive equals. */
      boolean cEq;
      
      /** Expected test results for case-sensitive notEquals. */
      boolean cNe;
      
      /** Expected test results for case-sensitive isGreaterThan. */
      boolean cGt;
      
      /** Expected test results for case-sensitive isLessThan. */
      boolean cLt;
      
      /** Expected test results for case-sensitive isGreaterThanOrEqual. */
      boolean cGe;
      
      /** Expected test results for case-sensitive isLessThanOrEqual. */
      boolean cLe;
      
      /** Expected test results for case-sensitive matches. */
      boolean cMatches;
      
      TestCase(String op1 , String op2     , boolean begins , boolean eq,
               boolean ne , boolean gt     , boolean lt     , boolean ge,
               boolean le , boolean matches, boolean cBegins, boolean cEq,
               boolean cNe, boolean cGt    , boolean cLt    , boolean cGe,
               boolean cLe, boolean cMatches)
      {
         // simulate the resulting runtime strings
         String s1 = progressToJavaString("'" + op1 + "'");
         String s2 = progressToJavaString("'" + op2 + "'");
         
         this.op1      = new character(StringHelper.processEscapes(s1));
         this.op2      = new character(StringHelper.processEscapes(s2));
         this.begins   = begins;
         this.eq       = eq;
         this.ne       = ne;
         this.gt       = gt;
         this.lt       = lt;
         this.ge       = ge;
         this.le       = le;
         this.matches  = matches;
         this.cBegins  = cBegins;
         this.cEq      = cEq;
         this.cNe      = cNe;
         this.cGt      = cGt;
         this.cLt      = cLt;
         this.cGe      = cGe;
         this.cLe      = cLe;
         this.cMatches = cMatches;
      }
      
      boolean test()
      {
         boolean flag = true;
         
         if (character._begins(op1, op2) != begins)
         {
            flag = false;
            System.out.println("begins failure for \"" + op1 + "\" and \"" +
                               op2 + "\"");
         }
         op1.caseSens = true;
         if (character._begins(op1, op2) != cBegins)
         {
            flag = false;
            System.out.println("begins(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.equals(op1, op2) != eq)
         {
            flag = false;
            System.out.println("isEqual failure for \"" + op1 + "\" and \"" +
                               op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.equals(op1, op2) != cEq)
         {
            flag = false;
            System.out.println("isEqual(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.notEqual(op1, op2) != ne)
         {
            flag = false;
            System.out.println("notEqual failure for \"" + op1 + "\" and \"" +
                               op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.notEqual(op1, op2) != cNe)
         {
            flag = false;
            System.out.println("notEqual(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.greaterThan(op1, op2) != gt)
         {
            flag = false;
            System.out.println("isGreaterThan failure for \"" + op1 +
                               "\" and \"" + op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.greaterThan(op1, op2) != cGt)
         {
            flag = false;
            System.out.println("isGreaterThan(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.lessThan(op1, op2) != lt)
         {
            flag = false;
            System.out.println("isLessThan failure for \"" + op1 +
                               "\" and \"" + op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.lessThan(op1, op2) != cLt)
         {
            flag = false;
            System.out.println("isLessThan(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.greaterThanOrEqual(op1, op2) != ge)
         {
            flag = false;
            System.out.println("isGreaterThanOrEqual failure for \"" + op1 +
                               "\" and \"" + op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.greaterThanOrEqual(op1, op2) != cGe)
         {
            flag = false;
            System.out.println("isGreaterThanOrEqual(c) failure for \"" +
                                op1 + "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         if (CompareOps.lessThanOrEqual(op1, op2) != le)
         {
            flag = false;
            System.out.println("isLessThanOrEqual failure for \"" + op1 +
                               "\" and \"" + op2 + "\"");
         }
         op1.caseSens = true;
         if (CompareOps.lessThanOrEqual(op1, op2) != cLe)
         {
            flag = false;
            System.out.println("isLessThanOrEqual(c) failure for \"" + op1 +
                                "\" and \"" + op2 + "\"");
         }
         op1.caseSens = false;
         
         if (character._matches(op1, op2) != matches)
         {
            flag = false;
            System.out.println("matches failure for \"" + this.op1 + 
                               "\" (\"" + op1 + "\") and \"" + this.op2 + 
                               "\" (\"" + op2 + "\")");
         }
         op1.caseSens = true;
         if (character._matches(op1, op2) != cMatches)
         {
            flag = false;
            System.out.println("matches(c) failure for \"" + this.op1 +
                                "\" (\"" + op1 + "\") and \"" + this.op2 +
                                "\" (\"" + op2 + "\")");
         }
         
         return flag;
      }
   }
   
   /**
    * Command line interface which provides a test harness to regression
    * test this class.  No parameters are necessary.
    */
   public static void main(String[] args)
   {            
      // if still true after all tests, everything passed
      boolean flag = true;
           
      TestCase tests[] =
      {
         new TestCase(" hello "    , "hello"      , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase("hello"      , "hello  "    , false, true , false, false,
                                                    false, true , true , false,
                                                    false, true , false, false,
                                                    false, true , true , false),
         new TestCase("hello    "  , "hello"      , true , true , false, false,
                                                    false, true , true , false,
                                                    true , true , false, false,
                                                    false, true , true , false),
         new TestCase("hELLo "     , "hello"      , true , true , false, false,
                                                    false, true , true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase("hello    "  , "heLLo"      , true , true , false, false,
                                                    false, true , true , false,
                                                    false, false, true , true ,
                                                    false, true , false, false),
         new TestCase(" hello "    , ".ell."      , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" hello "    , ".*ell*"     , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" hello~hel", "..ell.\\*"   , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" hel~lo "    , " hell*"    , false, false, true , true ,
                                                    false, true , false, true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase(" ajaj\\aj " , "*aj*"       , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj\\aj " , "*aj\\aj "   , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj\\aj " , "*ajaj "     , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj\\\\aj " , "*ajaj "   , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj\\\\\\aj " , "*ajaj " , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj\\\\aj " , "*aj\\aj " , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj\\\\aj " , "*aj\\\\aj ", false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj\\\\aj ", " ajaj\\\\aj ", true , true , false, false,
                                                    false, true , true , false,
                                                    true , true , false, false,
                                                    false, true , true , false),
         new TestCase(" ajaj~aj " , "*aj*"        , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj~aj "   , "*aj~~aj "  , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj~aj "   , "*ajaj "    , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase(" ajaj~~aj "  , "*ajaj "    , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj~~~aj "  , "*ajaj "   , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj~~aj "  , "*aj~aj "   , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj~~aj "  , "*aj~~aj "  , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase(" ajaj~~aj "  , " ajaj~~aj ", true , true , false, false,
                                                    false, true , true , false,
                                                    true , true , false, false,
                                                    false, true , true , false),
         new TestCase("a\\a"         , "a~~a"     , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase("a\\a"         , "a\\\\a"   , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase("a\\\\a"       , "a\\\\\\\\a", false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase("a~~a"        , "a~~~~a"    , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase("*a.b"       , "~~*a~~.b"     , false, false, true , false,
                                                    true , false, true , true ,
                                                    false, false, true , false,
                                                    true , false, true , true ),
         new TestCase("*a.b"       , "~*a~.b"       , true , true , false, false,
                                                    false, true , true , true ,
                                                    true , true , false, false,
                                                    false, true , true , true ),
         new TestCase("*a.b"       , "*a.b"       , true , true , false, false,
                                                    false, true , true , true ,
                                                    true , true , false, false,
                                                    false, true , true , true ),
         new TestCase("*axb"       , "~~*a~~.b"     , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase("*axb"       , "~*a~.b"       , false, false, true , true ,
                                                    false, true , false, true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase("*axb"       , "*a.b"       , false, false, true , true ,
                                                    false, true , false, true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase("xa.b"       , "~~*a~~.b"     , false, false, true , false,
                                                    true , false, true , false,
                                                    false, false, true , false,
                                                    true , false, true , false),
         new TestCase("xa.b"       , "~*a~.b"       , false, false, true , true ,
                                                    false, true , false, true ,
                                                    false, false, true , true ,
                                                    false, true , false, true ),
         new TestCase("xa.b"       , "*a.b"       , false, false, true , true ,
                                                    false, true , false, true ,
                                                    false, false, true , true ,
                                                    false, true , false, true )
      };      
          
      // run the tests
      for (int i = 0; i < tests.length; i++)
      {
         if (tests[i].test() == false)
            flag = false;
      }
      
      if (flag)
      {
         System.out.println("All tests passed successfully.");
      }

      System.out.println(matchesList("abc,def", "ABc") + " yes");
      System.out.println(matchesList("abc,def", "dEF") + " yes");
      System.out.println(matchesList("abc,def", "xyz") + " no ");
      System.out.println(matchesList("!abc,def", "ABc") + " no");
      System.out.println(matchesList("!abc,def", "dEF") + " yes");
      System.out.println(matchesList("abc*,def", "ABc") + " yes");
      System.out.println(matchesList("ab*,def", "ABc")  + " yes");
      System.out.println(matchesList("a*b*c,def", "AxBc") + " yes");
      System.out.println(matchesList("a*b*c,def", "AxByc") + " yes");
      System.out.println(matchesList("!a*b*c,def", "AxBc") + " no ");
      System.out.println(matchesList("!a*b*c,def", "AxByc") + " no ");
      System.out.println(matchesList("a!bc,def", "A!Bc") + " yes");
      System.out.println(matchesList("abc,d!ef", "d!EF") + " yes");
      System.out.println(matchesList(",abc,def", "") + " yes");
      System.out.println(matchesList(" ,abc,def", "") + " no ");
      System.out.println(matchesList(" ,abc,def", " ") + " yes");
      System.out.println(matchesList("abc,def,", "") + " yes");
      System.out.println(matchesList("abc,def,", " ") + " no ");
      System.out.println(matchesList("abc,def, ", " ") + " yes");
      System.out.println(matchesList("", " ") + " no ");
      System.out.println(matchesList("", "") + " yes");
      System.out.println(matchesList(new character(), " ") + " ?  ");
      System.out.println(matchesList(" ", new character()) + " ?  ");
      System.out.println(matchesList(new character(), new character()) + " ?  ");
      System.out.println(matchesList(new character()) + " exception");

   }
   
   // This is the Progress 4GL program that was used to generate the test
   // case results.  As output it generates the Java statements to initialize
   // the tests[] array in main(), HOWEVER due to the natural conversion of
   // string literals by Progress (on input), the corresponding output
   // strings are missing tildes and backslashes... so you will have to fix
   // up these strings to include the missing data.  In addition, since Java
   // modifies string literals at compile time, you will need to double up
   // every backslash to ensure that the resulting number of backslashes
   // match what the Progress program used.  So if there were 2 backslashes
   // in the Progress version of a string, the Java version will need 4.
   // Once these fixups occur in the main() method, then the tests can be
   // run to determine the level of compatibility between this class and
   // Progress string comparison operations.
   /*   
   
   def var op1  as char extent 38.
   def var op1_ as char extent 38 case-sensitive.
   def var op2  as char extent 38.
   def var res  as logical format "true/false".
   def var i    as int.
   def var str  as char.
   
   op1[1]  = " hello ".     op2[1]  = "hello".
   op1[2]  = "hello".       op2[2]  = "hello  ".
   op1[3]  = "hello    ".   op2[3]  = "hello".    
   op1[4]  = "hELLo ".      op2[4]  = "hello".
   op1[5]  = "hello    ".   op2[5]  = "heLLo".
   op1[6]  = " hello ".     op2[6]  = ".ell.".
   op1[7]  = " hello ".     op2[7]  = ".*ell*".
   op1[8]  = " hello~hel".  op2[8]  = "..ell.\*".
   op1[9]  = " hel~lo ".    op2[9]  = " hell*".
   op1[10] = " ajaj\aj ".   op2[10] = "*aj*".
   op1[11] = " ajaj\aj ".   op2[11] =  "*aj\aj ".
   op1[12] = " ajaj\aj ".   op2[12] =  "*ajaj ".
   op1[13] = " ajaj\\aj ".  op2[13] =  "*ajaj ".
   op1[14] = " ajaj\\\aj ". op2[14] =  "*ajaj ".
   op1[15] = " ajaj\\aj ".  op2[15] =  "*aj\aj ".
   op1[16] = " ajaj\\aj ".  op2[16] =  "*aj\\aj ".
   op1[17] = " ajaj\\aj ".  op2[17] =  " ajaj\\aj ".
   op1[18] = " ajaj~aj ".   op2[18] =  "*aj*".
   op1[19] = " ajaj~aj ".   op2[19] =  "*aj~aj ".
   op1[20] = " ajaj~aj ".   op2[20] =  "*ajaj ".
   op1[21] = " ajaj~~aj ".  op2[21] =  "*ajaj ".
   op1[22] = " ajaj~~~aj ". op2[22] =  "*ajaj ".
   op1[23] = " ajaj~~aj ".  op2[23] =  "*aj~aj ".
   op1[24] = " ajaj~~aj ".  op2[24] =  "*aj~~aj ".
   op1[25] = " ajaj~~aj ".  op2[25] =  " ajaj~~aj ".
   op1[26] = "a\a".         op2[26] =  "a~~a".
   op1[27] = "a\a".         op2[27] =  "a\\a".
   op1[28] = "a\\a".        op2[28] =  "a\\\\a".
   op1[29] = "a~~a".        op2[29] =  "a~~~~a".
   op1[30] = "*a.b".        op2[30] =  "~~*a~~.b".
   op1[31] = "*a.b".        op2[31] =  "~*a~.b".
   op1[32] = "*a.b".        op2[32] =  "*a.b".
   op1[33] = "*axb".        op2[33] =  "~~*a~~.b".
   op1[34] = "*axb".        op2[34] =  "~*a~.b".
   op1[35] = "*axb".        op2[35] =  "*a.b".
   op1[36] = "xa.b".        op2[36] =  "~~*a~~.b".
   op1[37] = "xa.b".        op2[37] =  "~*a~.b".
   op1[38] = "xa.b".        op2[38] =  "*a.b".
   
   output to value("listing.txt").
   
   do i = 1 to 38.
      str = "         new TestCase(\"" + op1[i] + "\"" +
            fill(" ", 11 - length(op1[i])) + ", \"" + op2[i] + "\"" +
            fill(" ", 11 - length(op2[i])) + ", ". 
      put str format "x(52)".
   
      res = op1[i] begins op2[i].
      put res ", ".
      res = op1[i] eq op2[i].
      put res ", ".   
      res = op1[i] ne op2[i].
      put res ", ".
      res = op1[i] gt op2[i].
      put res ",~n".
      put fill(" ", 52) format "x(52)".
      res = op1[i] lt op2[i].
      put res ", ".   
      res = op1[i] ge op2[i].
      put res ", ".   
      res = op1[i] le op2[i].
      put res ", ".   
      res = op1[i] matches op2[i].
      put res ",~n".   
      put fill(" ", 52) format "x(52)".
      
      op1_[i] = op1[i].
      
      res = op1_[i] begins op2[i].
      put res ", ".
      res = op1_[i] eq op2[i].
      put res ", ".   
      res = op1_[i] ne op2[i].
      put res ", ".
      res = op1_[i] gt op2[i].
      put res ",~n".
      put fill(" ", 52) format "x(52)".
      res = op1_[i] lt op2[i].
      put res ", ".   
      res = op1_[i] ge op2[i].
      put res ", ".   
      res = op1_[i] le op2[i].
      put res ", ".   
      res = op1_[i] matches op2[i].
      put res "),~n".      
   end.
   
   output close.
   
   */   
}
